@@ -38,7 +38,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: fetchnews.c,v 1.14 2005/03/23 00:40:11 shadow Exp $
+ * $Id: fetchnews.c,v 1.9 2004/05/27 03:04:59 ken3 Exp $
  */
 
 #include <config.h>
@@ -125,7 +125,7 @@ int newsrc_done(void)
 void usage(void)
 {
     fprintf(stderr,
-	    "fetchnews [-C <altconfig>] [-s <server>] [-n] [-y] [-w <wildmat>] [-f <tstamp file>]\n"
+	    "fetchnews [-C <altconfig>] [-s <server>] [-n] [-w <wildmat>] [-f <tstamp file>]\n"
 	    "          [-a <authname> [-p <password>]] <peer>\n");
     exit(-1);
 }
@@ -172,15 +172,15 @@ int fetch(char *msgid, int bymsgid,
 	  struct protstream *sin, struct protstream *sout,
 	  int *rejected, int *accepted, int *failed)
 {
-    char buf[4096];
+    char buf[4096], sbuf[4096];
 
     /* see if we want this article */
     prot_printf(sout, "IHAVE %s\r\n", msgid);
-    if (!prot_fgets(buf, sizeof(buf), sin)) {
+    if (!prot_fgets(sbuf, sizeof(sbuf), sin)) {
 	syslog(LOG_ERR, "IHAVE terminated abnormally");
 	return -1;
     }
-    else if (strncmp("335", buf, 3)) {
+    else if (strncmp("335", sbuf, 3)) {
 	/* don't want it */
 	(*rejected)++;
 	return 0;
@@ -216,12 +216,6 @@ int fetch(char *msgid, int bymsgid,
 	    }
 
 	    do {
-		/* look for malformed lines with NUL CR LF */
-		if (buf[strlen(buf)-1] != '\n' &&
-		    strlen(buf)+2 < sizeof(buf)-1 &&
-		    buf[strlen(buf)+2] == '\n') {
-		    strlcat(buf, "\r\n", sizeof(buf));
-		}
 		prot_printf(sout, "%s", buf);
 	    } while (buf[strlen(buf)-1] != '\n' &&
 		     prot_fgets(buf, sizeof(buf), pin));
@@ -247,7 +241,6 @@ int fetch(char *msgid, int bymsgid,
 }
 
 #define RESP_GROW 100
-#define BUFFERSIZE 4096
 
 int main(int argc, char *argv[])
 {
@@ -258,17 +251,17 @@ int main(int argc, char *argv[])
     char *authname = NULL, *password = NULL;
     int psock = -1, ssock = -1;
     struct protstream *pin, *pout, *sin, *sout;
-    char buf[BUFFERSIZE];
+    char buf[4096];
     char sfile[1024] = "";
-    int fd = -1, i, n, offered, rejected, accepted, failed;
+    int fd, i, n, offered, rejected, accepted, failed;
     time_t stamp;
+    struct tm *tm;
     char **resp = NULL;
     int newnews = 1;
-    char *datefmt = "%y%m%d %H%M%S";
 
     if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
 
-    while ((opt = getopt(argc, argv, "C:s:w:f:a:p:n:y")) != EOF) {
+    while ((opt = getopt(argc, argv, "C:s:w:f:a:p:n")) != EOF) {
 	switch (opt) {
 	case 'C': /* alt config file */
 	    alt_config = optarg;
@@ -301,10 +294,6 @@ int main(int argc, char *argv[])
 	case 'n': /* no newnews */
 	    newnews = 0;
 	    break;
-	
-	case 'y': /* newsserver is y2k compliant */
-	    datefmt = "%Y%m%d %H%M%S";
-	    break;
 
 	default:
 	    usage();
@@ -371,24 +360,6 @@ int main(int argc, char *argv[])
     prot_fgets(buf, sizeof(buf), pin);
 
     if (newnews) {
-	struct tm ctime, *ptime;
-
-	/* fetch the server's current time */
-	prot_printf(pout, "DATE\r\n");
-
-	if (!prot_fgets(buf, sizeof(buf), pin) || strncmp("111 ", buf, 4)) {
-	    syslog(LOG_ERR, "error fetching DATE");
-	    goto quit;
-	}
-
-	/* parse and normalize the server time */
-	memset(&ctime, 0, sizeof(struct tm));
-	sscanf(buf+4, "%4d%02d%02d%02d%02d%02d",
-	       &ctime.tm_year, &ctime.tm_mon, &ctime.tm_mday,
-	       &ctime.tm_hour, &ctime.tm_min, &ctime.tm_sec);
-	ctime.tm_year -= 1900;
-	ctime.tm_mon--;
-
 	/* read the previous timestamp */
 	if (!sfile[0]) {
 	    char oldfile[1024];
@@ -415,9 +386,8 @@ int main(int argc, char *argv[])
 	}
 
 	/* ask for new articles */
-	if (stamp) stamp -= 180; /* adjust back 3 minutes */
-	ptime = gmtime(&stamp);
-	strftime(buf, sizeof(buf), datefmt, ptime);
+	tm = gmtime(&stamp);
+	strftime(buf, sizeof(buf), "%Y%m%d %H%M%S", tm);
 	prot_printf(pout, "NEWNEWS %s %s GMT\r\n", wildmat, buf);
 	
 	if (!prot_fgets(buf, sizeof(buf), pin) || strncmp("230", buf, 3)) {
@@ -425,15 +395,7 @@ int main(int argc, char *argv[])
 	    newnews = 0;
 	}
 
-	/* prepare server's current time as new timestamp */
-	stamp = mktime(&ctime);
-	/* adjust for local timezone
-
-	   XXX  We need to do this because we use gmtime() above.
-	   We can't change this, otherwise we'd be incompatible
-	   with an old localtime timestamp.
-	*/
-	stamp -= timezone;
+	stamp = time(NULL);
     }
 
     if (!newnews) {
@@ -502,7 +464,7 @@ int main(int argc, char *argv[])
 	close(fd);
     }
     else {
-	char group[BUFFERSIZE], msgid[BUFFERSIZE], lastbuf[50];
+	char group[1024], msgid[1024], lastbuf[50];
 	const char *data;
 	unsigned long low, high, last, cur;
 	int start;
