@@ -38,7 +38,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: idle_idled.c,v 1.16 2005/12/09 21:09:14 murch Exp $ */
+/* $Id: idle_idled.c,v 1.14 2003/10/22 20:05:11 ken3 Exp $ */
 
 #include <config.h>
 
@@ -65,7 +65,6 @@ static idle_updateproc_t *idle_update = NULL;
 
 /* how often to poll the mailbox */
 static time_t idle_period = -1;
-static int idle_started = 0;
 
 /* UNIX socket variables */
 static int notify_sock = -1;
@@ -137,9 +136,6 @@ int idle_enabled(void)
 
 static void idle_poll(int sig)
 {
-    /* ignore the signals, unless the server has started idling */
-    if (!idle_started) return;
-
     switch (sig) {
     case SIGUSR1:
 	idle_update(IDLE_MAILBOX);
@@ -154,47 +150,40 @@ static void idle_poll(int sig)
     }
 }
 
-int idle_init(idle_updateproc_t *proc)
+int idle_init(struct mailbox *mailbox, idle_updateproc_t *proc)
 {
     struct sigaction action;
 
     idle_update = proc;
 
-    /* We don't want recursive calls to idle_update() */
     sigemptyset(&action.sa_mask);
-    sigaddset(&action.sa_mask, SIGUSR1);
-    sigaddset(&action.sa_mask, SIGUSR2);
     action.sa_flags = 0;
 #ifdef SA_RESTART
     action.sa_flags |= SA_RESTART;
 #endif
     action.sa_handler = idle_poll;
 
-    /* Setup the signal handlers */
-    if ((sigaction(SIGUSR1, &action, NULL) < 0) ||
-	(sigaction(SIGUSR2, &action, NULL) < 0) ||
-	(sigaction(SIGALRM, &action, NULL) < 0)) {
-	syslog(LOG_ERR, "sigaction: %m");
+    /* Tell idled that we're idling */
+    if (idle_send_msg(IDLE_INIT, mailbox)) {
+	/* if we can talk to idled, setup the signal handlers */
+	if ((sigaction(SIGUSR1, &action, NULL) < 0) ||
+	    (sigaction(SIGUSR2, &action, NULL) < 0)) {
+	    syslog(LOG_ERR, "sigaction: %m");
+	    return 0;
+	}
+    }
+    else { /* otherwise, we'll poll with SIGALRM */
+	if (sigaction(SIGALRM, &action, NULL) < 0) {
+	    syslog(LOG_ERR, "sigaction: %m");
+	    return 0;
+	}
 
-	/* Cancel receiving signals */
-	idle_done(NULL);
-	return 0;
+	alarm(idle_period);
     }
 
     return 1;
 }
 
-void idle_start(struct mailbox *mailbox)
-{
-    idle_started = 1;
-
-    /* Tell idled that we're idling */
-    if (!idle_send_msg(IDLE_INIT, mailbox)) {
-	/* otherwise, we'll poll with SIGALRM */
-	alarm(idle_period);
-    }
-}
-
 void idle_done(struct mailbox *mailbox)
 {
     /* Tell idled that we're done idling */
@@ -204,9 +193,6 @@ void idle_done(struct mailbox *mailbox)
     signal(SIGUSR1, SIG_IGN);
     signal(SIGUSR2, SIG_IGN);
     signal(SIGALRM, SIG_IGN);
-
-    idle_update = NULL;
-    idle_started = 0;
 }
 
 /*
