@@ -1,5 +1,5 @@
 /* mboxname.c -- Mailbox list manipulation routines
- * $Id: mboxname.c,v 1.36 2005/02/14 06:39:57 shadow Exp $
+ * $Id: mboxname.c,v 1.34 2004/05/22 03:45:51 rjs3 Exp $
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -122,10 +122,8 @@ static int mboxname_tointernal(struct namespace *namespace, const char *name,
 	    userlen = cp++ - userid;
 	    /* don't prepend default domain */
 	    if (!(config_defdomain && !strcasecmp(config_defdomain, cp))) {
-		domainlen = strlen(cp)+1;
-		if (domainlen > MAX_MAILBOX_NAME) 
-		    return IMAP_MAILBOX_BADNAME; 
 		sprintf(result, "%s!", cp);
+		domainlen = strlen(result);
 	    }
 	}
 	if ((cp = strrchr(name, '@'))) {
@@ -146,10 +144,8 @@ static int mboxname_tointernal(struct namespace *namespace, const char *name,
 		       (except for global admin) */
 		    return IMAP_MAILBOX_BADNAME;
 		}
-		domainlen = strlen(cp+1)+1;
-		if (domainlen > MAX_MAILBOX_NAME) 
-		    return IMAP_MAILBOX_BADNAME; 
 		sprintf(result, "%s!", cp+1);
+		domainlen = strlen(result);
 	    }
 	}
 
@@ -198,7 +194,6 @@ static int mboxname_tointernal_alt(struct namespace *namespace,
     char *cp;
     int userlen, domainlen = 0, namelen;
     int prefixlen;
-    size_t resultlen;
 
     /* Blank the result, just in case */
     result[0] = '\0';
@@ -212,10 +207,8 @@ static int mboxname_tointernal_alt(struct namespace *namespace,
 	    userlen = cp++ - userid;
 	    if (!(config_defdomain && !strcasecmp(config_defdomain, cp))) {
 		/* don't prepend default domain */
-		domainlen = strlen(cp)+1;
-		if (domainlen > MAX_MAILBOX_NAME) 
-		    return IMAP_MAILBOX_BADNAME; 
 		sprintf(result, "%s!", cp);
+		domainlen = strlen(result);
 	    }
 	}
 	if ((cp = strrchr(name, '@'))) {
@@ -236,10 +229,8 @@ static int mboxname_tointernal_alt(struct namespace *namespace,
 		       (except for global admin) */
 		    return IMAP_MAILBOX_BADNAME;
 		}
-		domainlen = strlen(cp+1)+1;
-		if (domainlen > MAX_MAILBOX_NAME) 
-		    return IMAP_MAILBOX_BADNAME; 
 		sprintf(result, "%s!", cp+1);
+		domainlen = strlen(result);
 	    }
 	}
 
@@ -319,14 +310,11 @@ static int mboxname_tointernal_alt(struct namespace *namespace,
 	return 0;
     }
 
-    resultlen = strlen(result);
-
     /* other personal folder */
-    if (domainlen+resultlen+6+namelen > MAX_MAILBOX_NAME) {
+    if (domainlen+strlen(result)+6+namelen > MAX_MAILBOX_NAME) {
 	return IMAP_MAILBOX_BADNAME;
     }
-    snprintf(result+resultlen, MAX_MAILBOX_NAME+1-resultlen, ".%.*s", 
-	     namelen, name);
+    sprintf(result+strlen(result), ".%.*s", namelen, name);
 
     /* Translate any separators in mailboxname */
     mboxname_hiersep_tointernal(namespace, result+6+userlen, 0);
@@ -345,7 +333,7 @@ static int mboxname_toexternal(struct namespace *namespace, const char *name,
 			       const char *userid, char *result)
 {
     char *domain = NULL, *cp;
-    size_t domainlen = 0, resultlen;
+    size_t domainlen = 0;
 
     /* Blank the result, just in case */
     result[0] = '\0';
@@ -368,16 +356,9 @@ static int mboxname_toexternal(struct namespace *namespace, const char *name,
     /* Translate any separators in mailboxname */
     mboxname_hiersep_toexternal(namespace, result, 0);
 
-    resultlen = strlen(result);
-
     /* Append domain */
-    if (domain) {
-	if(resultlen+domainlen+1 > MAX_MAILBOX_NAME) 
-	    return IMAP_MAILBOX_BADNAME;
-
-	snprintf(result+resultlen, MAX_MAILBOX_NAME+1-resultlen, 
-		 "@%.*s", domainlen, domain);
-    }
+    if (domain)
+	sprintf(result+strlen(result), "@%.*s", domainlen, domain);
 
     return 0;
 }
@@ -387,7 +368,6 @@ static int mboxname_toexternal_alt(struct namespace *namespace, const char *name
 				  const char *userid, char *result)
 {
     char *domain;
-    size_t userlen, resultlen;
 
     /* Blank the result, just in case */
     result[0] = '\0';
@@ -396,18 +376,10 @@ static int mboxname_toexternal_alt(struct namespace *namespace, const char *name
     
     if (!userid) return IMAP_MAILBOX_BADNAME;
 
-    userlen = strlen(userid);
-
-    if (config_virtdomains && (domain = strchr(userid, '@'))) {
-	size_t domainlen = strlen(domain);
-
-	userlen = domain - userid;
-
-	if (!strncmp(name, domain+1, domainlen-1) &&
-	    name[domainlen-1] == '!') {
-	    name += domainlen;
-	}
-    }
+    if (config_virtdomains && (domain = strchr(userid, '@')) &&
+	!strncmp(name, domain+1, strlen(domain)-1) &&
+	name[strlen(domain)-1] == '!')
+	name += strlen(domain);
 
     /* Personal (INBOX) namespace */
     if (!strncasecmp(name, "inbox", 5) &&
@@ -419,30 +391,24 @@ static int mboxname_toexternal_alt(struct namespace *namespace, const char *name
     }
     /* paranoia - this shouldn't be needed */
     else if (!strncmp(name, "user.", 5) &&
-	     !strncmp(name+5, userid, userlen) &&
-	     (name[5+userlen] == '\0' ||
-	      name[5+userlen] == '.')) {
-	if (name[5+userlen] == '\0')
+	     !strncmp(name+5, userid, strlen(userid)) &&
+	     (name[5+strlen(userid)] == '\0' ||
+	      name[5+strlen(userid)] == '.')) {
+	if (name[5+strlen(userid)] == '\0')
 	    strcpy(result, "INBOX");
 	else
-	    strcpy(result, name+5+userlen+1);
+	    strcpy(result, name+5+strlen(userid)+1);
     }
 
     /* Other Users namespace */
     else if (!strncmp(name, "user", 4) &&
 	     (name[4] == '\0' || name[4] == '.')) {
-	size_t prefixlen = strlen(namespace->prefix[NAMESPACE_USER]);
-
-	if ((prefixlen > MAX_MAILBOX_NAME) || 
-	    ((name[4] == '.') && 
-	     ((prefixlen+1+strlen(name+5)) > MAX_MAILBOX_NAME)))
-	    return IMAP_MAILBOX_BADNAME;
-
 	sprintf(result, "%.*s",
-		prefixlen-1, namespace->prefix[NAMESPACE_USER]);
-	resultlen = strlen(result);
+		(int) strlen(namespace->prefix[NAMESPACE_USER])-1,
+		namespace->prefix[NAMESPACE_USER]);
 	if (name[4] == '.') {
-	    sprintf(result+resultlen, "%c%s", namespace->hier_sep, name+5);
+	    sprintf(result+strlen(result), "%c%s",
+		    namespace->hier_sep, name+5);
 	}
     }
 
