@@ -1,6 +1,6 @@
 /* mbpath.c -- help the sysadmin to find the path matching the mailbox
  *
- * $Id: mbpath.c,v 1.19 2006/02/25 18:31:51 murch Exp $
+ * $Id: mbpath.c,v 1.16 2004/05/26 15:32:10 rjs3 Exp $
  * 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
@@ -42,7 +42,7 @@
  *
  */
 
-/* static char _rcsid[] = "$Id: mbpath.c,v 1.19 2006/02/25 18:31:51 murch Exp $"; */
+/* static char _rcsid[] = "$Id: mbpath.c,v 1.16 2004/05/26 15:32:10 rjs3 Exp $"; */
 
 #include <config.h>
 
@@ -74,9 +74,6 @@
 extern int optind;
 extern char *optarg;
 
-/* current namespace */
-static struct namespace mbpath_namespace;
-
 /* config.c stuff */
 const int config_need_data = 0;
 
@@ -94,9 +91,6 @@ main(int argc, char **argv)
   int rc, i, quiet = 0, stop_on_error=0;
   int opt;		/* getopt() returns an int */
   char *alt_config = NULL;
-  char buf[MAX_MAILBOX_PATH+1];
-
-  if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
 
   while ((opt = getopt(argc, argv, "C:qs")) != EOF) {
     switch(opt) {
@@ -117,20 +111,12 @@ main(int argc, char **argv)
 
   cyrus_init(alt_config, "mbpath", 0);
 
-  if ((rc = mboxname_init_namespace(&mbpath_namespace, 1)) != 0) {
-    fatal(error_message(rc), -1);
-  }
-
   mboxlist_init(0);
   mboxlist_open(NULL);
 
   for (i = optind; i < argc; i++) {
     (void)memset(&path, 0, sizeof(path));
-
-    /* Translate mailboxname */
-    (*mbpath_namespace.mboxname_tointernal)(&mbpath_namespace, argv[i], NULL, buf);
-
-    if ((rc = mboxlist_lookup(buf, &path, NULL, NULL)) == 0) {
+    if ((rc = mboxlist_lookup(argv[i], &path, NULL, NULL)) == 0) {
       printf("%s\n", path);
     } else {
       if (!quiet && (rc == IMAP_MAILBOX_NONEXISTENT)) {
@@ -154,5 +140,5 @@ main(int argc, char **argv)
   return 0;
 }
 
-/* $Header: /cvs/src/cyrus/imap/mbpath.c,v 1.19 2006/02/25 18:31:51 murch Exp $ */
+/* $Header: /cvs/src/cyrus/imap/mbpath.c,v 1.16 2004/05/26 15:32:10 rjs3 Exp $ */
 
