@@ -1,7 +1,7 @@
 /* mupdate-client.c -- cyrus murder database clients
  *
- * $Id: mupdate-client.c,v 1.34 2002/11/19 19:32:31 rjs3 Exp $
- * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
+ * $Id: mupdate-client.c,v 1.44 2004/04/26 19:27:30 rjs3 Exp $
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -49,7 +49,7 @@
 #include <sasl/sasl.h>
 #include <sasl/saslutil.h>
 #include <syslog.h>
-#ifdef __STDC__
+#ifdef HAVE_STDARG_H
 #include <stdarg.h>
 #else
 #include <varargs.h>
@@ -58,241 +58,105 @@
 #include <unistd.h>
 #endif
 #include <sys/types.h>
-#include <sys/time.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
 
-#include "prot.h"
-#include "xmalloc.h"
-#include "imapconf.h"
 #include "assert.h"
+#include "cyrusdb.h"
+#include "exitcodes.h"
+#include "global.h"
 #include "imparse.h"
 #include "iptostring.h"
 #include "mupdate.h"
 #include "mupdate_err.h"
-#include "exitcodes.h"
+#include "prot.h"
+#include "protocol.h"
+#include "xmalloc.h"
 
 const char service_name[] = "mupdate";
 
-extern sasl_callback_t *mysasl_callbacks(const char *username,
-                                         const char *authname,
-                                         const char *realm,
-                                         const char *password);
-extern void free_callbacks(sasl_callback_t *in);
-
-static sasl_security_properties_t *make_secprops(int min, int max)
+static sasl_security_properties_t *make_secprops(void)
 {
   sasl_security_properties_t *ret =
       (sasl_security_properties_t *) xzmalloc(sizeof(sasl_security_properties_t));
 
   ret->maxbufsize = PROT_BUFSIZE;
-  ret->min_ssf = config_getint("sasl_minimum_layer", min);	
-  ret->max_ssf = config_getint("sasl_maximum_layer", max);
+  ret->min_ssf = config_getint(IMAPOPT_SASL_MINIMUM_LAYER);	
+  ret->max_ssf = config_getint(IMAPOPT_SASL_MAXIMUM_LAYER);
 
   return ret;
 }
 
-static int mupdate_authenticate(mupdate_handle *h,
-				const char *mechlist)
-{
-    int saslresult;
-    sasl_security_properties_t *secprops=NULL;
-    socklen_t addrsize;
-    struct sockaddr_in saddr_l;
-    struct sockaddr_in saddr_r;
-    char localip[60], remoteip[60];
-    const char *out;
-    unsigned int outlen;
-    const char *mechusing;
-    int ch;
-    char buf[4096];
-
-    /* Why do this again? */
-    if (h->saslcompleted) {
-	return 1;
-    }
-
-    secprops = make_secprops(0, 256);
-    if(!secprops) return 1;
-    
-    saslresult=sasl_setprop(h->saslconn, SASL_SEC_PROPS, secprops);
-    if(saslresult != SASL_OK) return 1;
-    free(secprops);
-    
-    addrsize=sizeof(struct sockaddr_in);
-    if (getpeername(h->sock,(struct sockaddr *)&saddr_r,&addrsize)!=0)
-	return 1;
-
-    addrsize=sizeof(struct sockaddr_in);
-    if (getsockname(h->sock,(struct sockaddr *)&saddr_l,&addrsize)!=0)
-	return 1;
-
-    if(iptostring((const struct sockaddr *)&saddr_l, sizeof(struct sockaddr_in),
-		  localip, 60) != 0)
-	return 1;
-    
-    if(iptostring((const struct sockaddr *)&saddr_r, sizeof(struct sockaddr_in),
-		  remoteip, 60) != 0)
-	return 1;
-
-    saslresult=sasl_setprop(h->saslconn, SASL_IPREMOTEPORT, remoteip);
-    if (saslresult!=SASL_OK) return 1;
-
-    saslresult=sasl_setprop(h->saslconn, SASL_IPLOCALPORT, localip);
-    if (saslresult!=SASL_OK) return 1;
-
-    /* We shouldn't get sasl_interact's,
-     * because we provide explicit callbacks */
-    saslresult = sasl_client_start(h->saslconn, mechlist,
-				   NULL, &out, &outlen, &mechusing);
-
-    if(saslresult != SASL_OK && saslresult != SASL_CONTINUE) return 1;
-
-    if(out) {
-	int r = sasl_encode64(out, outlen,
-			      buf, sizeof(buf), NULL);
-	if(r != SASL_OK) return 1;
-	
-	/* it's always ok to send the mechname quoted */
-	prot_printf(h->pout, "A01 AUTHENTICATE \"%s\" {%d+}\r\n%s\r\n",
-		    mechusing, strlen(buf), buf);
-    } else {
-        prot_printf(h->pout, "A01 AUTHENTICATE \"%s\"\r\n", mechusing);
-    }
-
-    while(saslresult == SASL_CONTINUE) {
-	char *p, *in;
-	unsigned int len, inlen;
-	
-	if(!prot_fgets(buf, sizeof(buf)-1, h->pin)) {
-	    /* Connection Dropped */
-	    return 1;
-	}
-
-	p = buf + strlen(buf) - 1;
-	if(p >= buf && *p == '\n') *p-- = '\0';
-	if(p >= buf && *p == '\r') *p-- = '\0';
-
-	len = strlen(buf);
-	in = xmalloc(len);
-	saslresult = sasl_decode64(buf, len, in, len, &inlen);
-	if(saslresult != SASL_OK) {
-	    free(in);
-
-	    /* CANCEL */
-	    syslog(LOG_ERR, "couldn't base64 decode: aborted authentication");
-
-	    /* If we haven't already canceled due to bad authentication,
-	     * then we should */
-	    if(strncmp(buf, "A01 NO ", 7)) prot_printf(h->pout, "*");
-	    else {
-		syslog(LOG_ERR,
-		       "Authentication to master failed (%s)", buf+7);
-	    }
-	    return 1;
-	}
-
-	saslresult = sasl_client_step(h->saslconn, in, inlen, NULL,
-				      &out, &outlen);
-	free(in);
-
-	if((saslresult == SASL_OK || saslresult == SASL_CONTINUE)) {
-	    int r = sasl_encode64(out, outlen,
-				  buf, sizeof(buf), NULL);
-	    if(r != SASL_OK) return 1;
-	    
-	    prot_printf(h->pout, "%s\r\n", buf);
-	}
-    }
-
-    if(saslresult != SASL_OK) {
-	syslog(LOG_ERR, "bad authentication: %s",
-	       sasl_errdetail(h->saslconn));
-	
-	prot_printf(h->pout, "*");
-	return 1;
-    }
-
-    /* Check Result */
-    ch = getword(h->pin, &(h->tag));
-    if(ch != ' ') return 1; /* need an OK or NO */
-
-    ch = getword(h->pin, &(h->cmd));
-    if(!strncmp(h->cmd.s, "NO", 2)) {
-	if(ch != ' ') return 1; /* no reason really necessary, but we failed */
-	ch = getstring(h->pin, h->pout, &(h->arg1));
-	syslog(LOG_ERR, "authentication failed: %s", h->arg1.s);
-	return 1;
-    }
-
-    prot_setsasl(h->pin, h->saslconn);
-    prot_setsasl(h->pout, h->saslconn);
-
-    h->saslcompleted = 1;
-
-    return 0; /* SUCCESS */
-}
-
 int mupdate_connect(const char *server, const char *port,
 		    mupdate_handle **handle,
 		    sasl_callback_t *cbs)
 {
     mupdate_handle *h = NULL;
+    struct addrinfo hints, *res0, *res;
+    int err = 0;
     int local_cbs = 0;
-    struct hostent *hp;
-    struct servent *sp;
-    struct sockaddr_in addr;
     int s, saslresult;
+    const char *proterr;
     char buf[4096];
     char *mechlist = NULL;
+    sasl_security_properties_t *secprops = NULL;
+    socklen_t addrsize;
+    struct sockaddr_storage saddr_l;
+    struct sockaddr_storage saddr_r;
+    char localip[60], remoteip[60];
+    const char *sasl_status = NULL;
+    char portstr[NI_MAXSERV];
+    const char *forcemech;
     
     if(!handle)
 	return MUPDATE_BADPARAM;
 
     /* open connection to 'server' */
     if(!server) {
-	server = config_getstring("mupdate_server", NULL);
+	server = config_mupdate_server;
 	if (server == NULL) {
 	    fatal("couldn't get mupdate server name", EC_UNAVAILABLE);
 	}
     }
-    if(!port) {
-	port = config_getstring("mupdate_port",NULL);
-    }
     
-    hp = gethostbyname(server);
-    if (!hp) {
-	syslog(LOG_ERR, "mupdate-client: gethostbyname %s failed: %m", server);
-	return MUPDATE_NOCONN;
-    }
-    
-    s = socket(AF_INET, SOCK_STREAM, 0);
-    if (s == -1) {
-	syslog(LOG_ERR, "mupdate-client: socket(): %m");
-	return MUPDATE_NOCONN;
-    }
-    
-    addr.sin_family = AF_INET;
-    memcpy(&addr.sin_addr, hp->h_addr, sizeof(addr.sin_addr));
-
-    if (port && imparse_isnumber(port)) {
-	addr.sin_port = htons(atoi(port));
-    } else if (port) {
-	sp = getservbyname(port, "tcp");
-	if (!sp) {
-	    syslog(LOG_ERR, "mupdate-client: getservbyname(tcp, %s): %m",
-		   port);
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    if(port)
+	err = getaddrinfo(server, port, &hints, &res0);
+    if (!port || err == EAI_SERVICE) {
+	err = getaddrinfo(server, "mupdate", &hints, &res0);
+	if (err == EAI_SERVICE) {
+	    snprintf(portstr, sizeof(portstr), "%d",
+		     config_getint(IMAPOPT_MUPDATE_PORT));
+	    err = getaddrinfo(server, portstr, &hints, &res0);
 	}
-	addr.sin_port = sp->s_port;
-    } else if((sp = getservbyname("mupdate", "tcp")) != NULL) {
-	addr.sin_port = sp->s_port;
-    } else {
-	addr.sin_port = htons(2004);
+    }
+    
+    if (err) {
+	syslog(LOG_ERR, "mupdate-client: getaddrinfo(%s, %s) failed: %s",
+	       server, port, gai_strerror(err));
+	return MUPDATE_NOCONN;
+    }
+    
+    s = -1;
+    for (res = res0; res; res = res->ai_next) {
+	s = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+	if (s < 0)
+	    continue;
+	if (connect(s, res->ai_addr, res->ai_addrlen) >= 0)
+	    break;
+	close(s);
+	s = -1;
     }
 
-    if (connect(s, (struct sockaddr *)&addr, sizeof(addr)) == -1) {
+    freeaddrinfo(res0);
+    if (s < 0) {
 	syslog(LOG_ERR, "mupdate-client: connect(%s): %m", server);
 	return MUPDATE_NOCONN;
     }
@@ -303,19 +167,12 @@ int mupdate_connect(const char *server, const char *port,
 
     if(!cbs) {
 	local_cbs = 1;
-	cbs = mysasl_callbacks(config_getstring("mupdate_username",""),
-			       config_getstring("mupdate_authname",NULL),
-			       config_getstring("mupdate_realm",NULL),
-			       config_getstring("mupdate_password",NULL));
+	cbs = mysasl_callbacks(config_getstring(IMAPOPT_MUPDATE_USERNAME),
+			       config_getstring(IMAPOPT_MUPDATE_AUTHNAME),
+			       config_getstring(IMAPOPT_MUPDATE_REALM),
+			       config_getstring(IMAPOPT_MUPDATE_PASSWORD));
     }
 
-    saslresult = sasl_client_new(service_name,
-				 server,
-				 NULL, NULL,
-				 cbs,
-				 0,
-				 &(h->saslconn));
-
     /* create protstream */
     h->pin = prot_new(h->sock, 0);
     h->pout = prot_new(h->sock, 1);
@@ -323,6 +180,38 @@ int mupdate_connect(const char *server, const char *port,
     prot_setflushonread(h->pin, h->pout);
     prot_settimeout(h->pin, 30*60);
 
+    /* set the IP addresses */
+    addrsize=sizeof(struct sockaddr_storage);
+    if (getpeername(h->sock,(struct sockaddr *)&saddr_r,&addrsize)!=0)
+	goto noconn;
+
+    addrsize=sizeof(struct sockaddr_storage);
+    if (getsockname(h->sock,(struct sockaddr *)&saddr_l,&addrsize)!=0)
+	goto noconn;
+
+    if(iptostring((const struct sockaddr *)&saddr_l, addrsize,
+		  localip, 60) != 0)
+	goto noconn;
+    
+    if(iptostring((const struct sockaddr *)&saddr_r, addrsize,
+		  remoteip, 60) != 0)
+	goto noconn;
+
+    saslresult = sasl_client_new(service_name,
+				 server,
+				 localip, remoteip,
+				 cbs,
+				 0,
+				 &(h->saslconn));
+    if(saslresult != SASL_OK) goto noconn;
+
+    secprops = make_secprops();
+    if(!secprops) goto noconn;
+    
+    saslresult=sasl_setprop(h->saslconn, SASL_SEC_PROPS, secprops);
+    if(saslresult != SASL_OK) goto noconn;
+    free(secprops);
+
     /* Read the mechlist & other capabilities */
     while(1) {
 	if (!prot_fgets(buf, sizeof(buf)-1, h->pin)) {
@@ -343,8 +232,17 @@ int mupdate_connect(const char *server, const char *port,
 	return MUPDATE_NOAUTH;
     }
     
-    if (mupdate_authenticate(h, mechlist)) {
-	syslog(LOG_ERR, "authentication to remote mupdate server failed");
+    forcemech = config_getstring(IMAPOPT_FORCE_SASL_CLIENT_MECH);
+    if(forcemech) {
+	free(mechlist);
+	mechlist = xstrdup(forcemech);
+    }
+
+    if (h->saslcompleted || 
+	saslclient(h->saslconn, &protocol[PROTOCOL_MUPDATE].sasl_cmd,
+		   mechlist, h->pin, h->pout, NULL, &sasl_status) != SASL_OK) {
+	syslog(LOG_ERR, "authentication to remote mupdate server failed: %s",
+	       sasl_status ? sasl_status : "already authenticated");
 	free(mechlist);
 	mupdate_disconnect(handle);
 	free_callbacks(cbs);
@@ -356,13 +254,20 @@ int mupdate_connect(const char *server, const char *port,
     /* xxx unclear that this is correct, but it prevents a memory leak */
     if(local_cbs) free_callbacks(cbs);
     
+    prot_setsasl(h->pin, h->saslconn);
+    prot_setsasl(h->pout, h->saslconn);
+
+    h->saslcompleted = 1;
+
     /* SUCCESS */
     return 0;
 
  noconn:
     if(mechlist) free(mechlist);
+    if(secprops) free(secprops);
+    proterr = prot_error(h->pin);
     syslog(LOG_ERR, "mupdate-client: connection to server closed: %s",
-	   prot_error(h->pin));
+	   proterr ? proterr : "(unknown)");
     mupdate_disconnect(handle);
 
     return MUPDATE_NOCONN;
@@ -397,36 +302,22 @@ void mupdate_disconnect(mupdate_handle **hp)
     *hp = NULL;
 }
 
-/* We're really only looking for an OK or NO or BAD here */
+/* We're really only looking for an OK or NO or BAD here -- and the callback
+ * is never called in those cases.  So if the callback is called, we have
+ * an error! */
 static int mupdate_scarf_one(struct mupdate_mailboxdata *mdata __attribute__((unused)),
 			     const char *cmd,
-			     void *context) 
+			     void *context __attribute__((unused))) 
 {
-    int *called = context;
-    
-    if(*called) {
-	/* Only want to be called once per command */
-	return -1;
-    }
-    *called = 1;
-
-    /*only accept OK, NO and BAD */
-    if(strncmp(cmd, "OK", 2)) {
-	return 0;
-    } else if (strncmp(cmd, "NO", 2) || strncmp(cmd, "BAD", 3)) {
-	return -1;
-    } else {
-	return 1;
-    }
+    syslog(LOG_ERR, "mupdate_scarf_one was called, but shouldn't be.  Command recieved was %s", cmd);
+    return -1;
 }
 
-
 int mupdate_activate(mupdate_handle *handle, 
 		     const char *mailbox, const char *server,
 		     const char *acl)
 {
     int ret;
-    int called = 0;
     enum mupdate_cmd_response response;
     
     if (!handle) return MUPDATE_BADPARAM;
@@ -438,7 +329,7 @@ int mupdate_activate(mupdate_handle *handle,
 		handle->tagn++, strlen(mailbox), mailbox, 
 		strlen(server), server, strlen(acl), acl);
 
-    ret = mupdate_scarf(handle, mupdate_scarf_one, &called, 1, &response);
+    ret = mupdate_scarf(handle, mupdate_scarf_one, NULL, 1, &response);
     if (ret) {
 	return ret;
     } else if (response != MUPDATE_OK) {
@@ -452,7 +343,6 @@ int mupdate_reserve(mupdate_handle *handle,
 		    const char *mailbox, const char *server)
 {
     int ret;
-    int called = 0;
     enum mupdate_cmd_response response;
     
     if (!handle) return MUPDATE_BADPARAM;
@@ -464,7 +354,7 @@ int mupdate_reserve(mupdate_handle *handle,
 		handle->tagn++, strlen(mailbox), mailbox, 
 		strlen(server), server);
 
-    ret = mupdate_scarf(handle, mupdate_scarf_one, &called, 1, &response);
+    ret = mupdate_scarf(handle, mupdate_scarf_one, NULL, 1, &response);
     if (ret) {
 	return ret;
     } else if (response != MUPDATE_OK) {
@@ -478,7 +368,6 @@ int mupdate_deactivate(mupdate_handle *handle,
 		       const char *mailbox, const char *server)
 {
     int ret;
-    int called = 0;
     enum mupdate_cmd_response response;
     
     if (!handle) return MUPDATE_BADPARAM;
@@ -490,7 +379,7 @@ int mupdate_deactivate(mupdate_handle *handle,
 		handle->tagn++, strlen(mailbox), mailbox, 
 		strlen(server), server);
 
-    ret = mupdate_scarf(handle, mupdate_scarf_one, &called, 1, &response);
+    ret = mupdate_scarf(handle, mupdate_scarf_one, NULL, 1, &response);
     if (ret) {
 	return ret;
     } else if (response != MUPDATE_OK) {
@@ -504,7 +393,6 @@ int mupdate_delete(mupdate_handle *handle,
 		   const char *mailbox)
 {
     int ret;
-    int called = 0;
     enum mupdate_cmd_response response;
     
     if (!handle) return MUPDATE_BADPARAM;
@@ -515,7 +403,7 @@ int mupdate_delete(mupdate_handle *handle,
 		"X%u DELETE {%d+}\r\n%s\r\n", handle->tagn++, 
 		strlen(mailbox), mailbox);
 
-    ret = mupdate_scarf(handle, mupdate_scarf_one, &called, 1, &response);
+    ret = mupdate_scarf(handle, mupdate_scarf_one, NULL, 1, &response);
     if (ret) {
 	return ret;
     } else if (response != MUPDATE_OK) {
@@ -679,22 +567,33 @@ int mupdate_scarf(mupdate_handle *handle,
 	if (wait_for_ok) {
 	    prot_BLOCK(handle->pin);
 	} else {
+	    /* check for input */
 	    prot_NONBLOCK(handle->pin);
+	    ch = prot_getc(handle->pin);
+
+	    if(ch == EOF && errno == EAGAIN) {
+		/* this was just "no input" we return 0 */
+		goto done;
+	    } else if(ch == EOF) {
+		/* this was a fatal error */
+		r = MUPDATE_NOCONN;
+		goto done;
+	    } else {
+		/* there's input waiting, put back our character */
+		prot_ungetc(ch, handle->pin);
+	    }
+
+	    /* Set it back to blocking so we don't get half a word */
+	    prot_BLOCK(handle->pin);
 	}
 
 	ch = getword(handle->pin, &(handle->tag));
-	if (ch == EOF && errno == EAGAIN) {
-	    /* this was just "no input" we return 0 */
-	    goto done;
-	} else if (ch == EOF) {
+	if (ch == EOF) {
 	    /* this was a fatal error */
 	    r = MUPDATE_NOCONN;
 	    goto done;
 	}
 
-	/* set it blocking so we don't get half a line */
-	prot_BLOCK(handle->pin);
-
 	if(ch != ' ') {
 	    /* We always have a command */
 	    syslog(LOG_ERR, "Protocol error from master: no tag");
@@ -719,15 +618,24 @@ int mupdate_scarf(mupdate_handle *handle,
 	
 	switch(handle->cmd.s[0]) {
 	case 'B':
-	    if(!strncmp(handle->cmd.s, "BAD", 6)) {
+	    if(!strncmp(handle->cmd.s, "BAD", 3)) {
 		ch = getstring(handle->pin, handle->pout, &(handle->arg1));
 		CHECKNEWLINE(handle, ch);
 
-		syslog(LOG_DEBUG, "mupdate BAD response: %s", handle->arg1.s);
+		syslog(LOG_ERR, "mupdate BAD response: %s", handle->arg1.s);
 		if (wait_for_ok && response) {
 		    *response = MUPDATE_BAD;
 		}
 		goto done;
+	    } else if (!strncmp(handle->cmd.s, "BYE", 3)) {
+		ch = getstring(handle->pin, handle->pout, &(handle->arg1));
+		CHECKNEWLINE(handle, ch);
+		
+		syslog(LOG_ERR, "mupdate BYE response: %s", handle->arg1.s);
+		if(wait_for_ok && response) {
+		    *response = MUPDATE_BYE;
+		}
+		goto done;
 	    }
 	    goto badcmd;
 
@@ -785,7 +693,7 @@ int mupdate_scarf(mupdate_handle *handle,
 	    }
 	    goto badcmd;
 	case 'N':
-	    if(!strncmp(handle->cmd.s, "NO", 6)) {
+	    if(!strncmp(handle->cmd.s, "NO", 2)) {
 		ch = getstring(handle->pin, handle->pout, &(handle->arg1));
 		CHECKNEWLINE(handle, ch);
 
@@ -854,3 +762,39 @@ int mupdate_scarf(mupdate_handle *handle,
 
     return r;
 }
+
+void kick_mupdate(void)
+{
+    char buf[2048];
+    struct sockaddr_un srvaddr;
+    int s, r;
+    int len;
+    
+    s = socket(AF_UNIX, SOCK_STREAM, 0);
+    if (s == -1) {
+	syslog(LOG_ERR, "socket: %m");
+	return;
+    }
+
+    strlcpy(buf, config_dir, sizeof(buf));
+    strlcat(buf, FNAME_MUPDATE_TARGET_SOCK, sizeof(buf));
+    memset((char *)&srvaddr, 0, sizeof(srvaddr));
+    srvaddr.sun_family = AF_UNIX;
+    strcpy(srvaddr.sun_path, buf);
+    len = sizeof(srvaddr.sun_family) + strlen(srvaddr.sun_path) + 1;
+
+    r = connect(s, (struct sockaddr *)&srvaddr, len);
+    if (r == -1) {
+	syslog(LOG_ERR, "kick_mupdate: can't connect to target: %m");
+	goto done;
+    }
+
+    r = read(s, buf, sizeof(buf));
+    if (r <= 0) {
+	syslog(LOG_ERR, "kick_mupdate: can't read from target: %m");
+    }
+
+ done:
+    close(s);
+    return;
+}
