@@ -40,7 +40,7 @@
  */
 
 /*
- * $Id: pop3d.c,v 1.167 2005/04/11 06:57:35 shadow Exp $
+ * $Id: pop3d.c,v 1.157 2004/05/22 03:45:52 rjs3 Exp $
  */
 #include <config.h>
 
@@ -109,8 +109,6 @@ sasl_conn_t *popd_saslconn; /* the sasl connection context */
 
 char *popd_userid = 0;
 struct mailbox *popd_mailbox = 0;
-struct auth_state *popd_authstate = 0;
-int config_popuseacl;
 struct sockaddr_storage popd_localaddr, popd_remoteaddr;
 int popd_haveaddr = 0;
 char popd_clienthost[NI_MAXHOST*2+1] = "[local]";
@@ -134,7 +132,7 @@ static mailbox_decideproc_t expungedeleted;
 
 /* the sasl proxy policy context */
 static struct proxy_context popd_proxyctx = {
-    0, 1, &popd_authstate, NULL, NULL
+    0, 1, NULL, NULL, NULL
 };
 
 /* signal to config.c */
@@ -173,7 +171,7 @@ extern void proc_cleanup(void);
 
 extern int saslserver(sasl_conn_t *conn, const char *mech,
 		      const char *init_resp, const char *resp_prefix,
-		      const char *continuation, const char *empty_chal,
+		      const char *continuation,
 		      struct protstream *pin, struct protstream *pout,
 		      int *sasl_result, char **success_data);
 
@@ -233,7 +231,9 @@ static void popd_reset(void)
     }
 #endif
 
-    cyrus_reset_stdio(); 
+    cyrus_close_sock(0);
+    cyrus_close_sock(1);
+    cyrus_close_sock(2);
 
     strcpy(popd_clienthost, "[local]");
     if (popd_logfd != -1) {
@@ -244,10 +244,6 @@ static void popd_reset(void)
 	free(popd_userid);
 	popd_userid = NULL;
     }
-    if (popd_authstate) {
-	auth_freestate(popd_authstate);
-	popd_authstate = NULL;
-    }
     if (popd_saslconn) {
 	sasl_dispose(&popd_saslconn);
 	popd_saslconn = NULL;
@@ -287,6 +283,7 @@ int service_init(int argc __attribute__((unused)),
 
     /* set signal handlers */
     signals_set_shutdown(&shut_down);
+    signals_add_handlers();
     signal(SIGPIPE, SIG_IGN);
 
     /* load the SASL plugins */
@@ -686,18 +683,7 @@ static void cmdloop(void)
 	    if (!arg) {
 		if (popd_mailbox) {
 		    if (!mailbox_lock_index(popd_mailbox)) {
-		        int pollpadding =config_getint(IMAPOPT_POPPOLLPADDING);
-			int minpollsec = config_getint(IMAPOPT_POPMINPOLL)*60;
-		        if ((minpollsec > 0) && (pollpadding > 1)) { 
-			    int mintime = popd_login_time - (minpollsec*(pollpadding));
-			    if (popd_mailbox->pop3_last_login < mintime) {
-			        popd_mailbox->pop3_last_login = mintime + minpollsec; 
-			    } else {
-			        popd_mailbox->pop3_last_login += minpollsec;
-			    }
-		        } else { 
-			    popd_mailbox->pop3_last_login = popd_login_time;
-		        }
+			popd_mailbox->pop3_last_login = popd_login_time;
 			mailbox_write_index_header(popd_mailbox);
 			mailbox_unlock_index(popd_mailbox);
 		    }
@@ -811,10 +797,6 @@ static void cmdloop(void)
 	}
 	else if (!strcmp(inputbuf, "dele")) {
 	    if (!arg) prot_printf(popd_out, "-ERR Missing argument\r\n");
-	    else if (config_popuseacl && !(mboxstruct.myrights & ACL_DELETE)) {
-		prot_printf(popd_out, "-ERR [SYS/PERM] %s\r\n",
-			    error_message(IMAP_PERMISSION_DENIED));
-	    }
 	    else {
 		msg = parsenum(&arg);
 		if (arg) {
@@ -1058,8 +1040,6 @@ static void cmd_apop(char *response)
     syslog(LOG_NOTICE, "login: %s %s APOP%s %s", popd_clienthost,
 	   popd_userid, popd_starttls_done ? "+TLS" : "", "User logged in");
 
-    popd_authstate = auth_newstate(popd_userid);
-
     openinbox();
 }
 
@@ -1166,8 +1146,6 @@ void cmd_pass(char *pass)
 	}
     }
 
-    popd_authstate = auth_newstate(popd_userid);
-
     openinbox();
 }
 
@@ -1272,8 +1250,8 @@ void cmd_auth(char *arg)
 	arg = NULL;
     }
 
-    r = saslserver(popd_saslconn, authtype, arg, "", "+ ", "",
-		   popd_in, popd_out, &sasl_result, NULL);
+    r = saslserver(popd_saslconn, authtype, arg, "", "+ ", popd_in, popd_out,
+		   &sasl_result, NULL);
 
     if (r) {
 	const char *errorstring = NULL;
@@ -1344,9 +1322,9 @@ void cmd_auth(char *arg)
 int openinbox(void)
 {
     char userid[MAX_MAILBOX_NAME+1], inboxname[MAX_MAILBOX_PATH+1];
-    int type, myrights = 0;
-    char *server = NULL, *acl;
-    int r, log_level = LOG_ERR;
+    int type;
+    char *server = NULL;
+    int r;
     const char *statusline = NULL;
 
     /* Translate any separators in userid
@@ -1359,21 +1337,11 @@ int openinbox(void)
     r = (*popd_namespace.mboxname_tointernal)(&popd_namespace, "INBOX",
 					      userid, inboxname);
 
-    if (!r) r = mboxlist_detail(inboxname, &type, NULL, &server, &acl, NULL);
-    if (!r && (config_popuseacl = config_getswitch(IMAPOPT_POPUSEACL)) &&
-	(!acl ||
-	 !((myrights = cyrus_acl_myrights(popd_authstate, acl)) & ACL_READ))) {
-	r = (myrights & ACL_LOOKUP) ?
-	    IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
-	log_level = LOG_INFO;
-    }
+    if (!r) r = mboxlist_detail(inboxname, &type, NULL, &server, NULL, NULL);
     if (r) {
 	sleep(3);
-	syslog(log_level, "Unable to locate maildrop for %s: %s",
-	       popd_userid, error_message(r));
-	prot_printf(popd_out,
-		    "-ERR [SYS/PERM] Unable to locate maildrop: %s\r\n",
-		    error_message(r));
+	syslog(LOG_ERR, "Unable to locate maildrop for %s", popd_userid);
+	prot_printf(popd_out, "-ERR [SYS/PERM] Unable to locate maildrop\r\n");
 	goto fail;
     }
 
@@ -1405,27 +1373,14 @@ int openinbox(void)
 	int msg;
 	struct index_record record;
 	int minpoll;
-	int doclose = 0;
 
 	popd_login_time = time(0);
 
-	r = mailbox_open_header(inboxname, popd_authstate, &mboxstruct);
-	if (!r) {
-	    doclose = 1;
-	    if (config_popuseacl && !(mboxstruct.myrights & ACL_READ)) {
-		r = (mboxstruct.myrights & ACL_LOOKUP) ?
-		    IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
-		log_level = LOG_INFO;
-	    }
-	}
+	r = mailbox_open_header(inboxname, 0, &mboxstruct);
 	if (r) {
 	    sleep(3);
-	    syslog(log_level, "Unable to open maildrop for %s: %s",
-		   popd_userid, error_message(r));
-	    prot_printf(popd_out,
-			"-ERR [SYS/PERM] Unable to open maildrop: %s\r\n",
-			error_message(r));
-	    if (doclose) mailbox_close(&mboxstruct);
+	    syslog(LOG_ERR, "Unable to open maildrop for %s", popd_userid);
+	    prot_printf(popd_out, "-ERR [SYS/PERM] Unable to open maildrop\r\n");
 	    goto fail;
 	}
 
@@ -1433,11 +1388,8 @@ int openinbox(void)
 	if (!r) r = mailbox_lock_pop(&mboxstruct);
 	if (r) {
 	    mailbox_close(&mboxstruct);
-	    syslog(LOG_ERR, "Unable to lock maildrop for %s: %s",
-		   popd_userid, error_message(r));
-	    prot_printf(popd_out,
-			"-ERR [IN-USE] Unable to lock maildrop: %s\r\n",
-			error_message(r));
+	    syslog(LOG_ERR, "Unable to lock maildrop for %s", popd_userid);
+	    prot_printf(popd_out, "-ERR [IN-USE] Unable to lock maildrop\r\n");
 	    goto fail;
 	}
 
@@ -1495,8 +1447,6 @@ int openinbox(void)
   fail:
     free(popd_userid);
     popd_userid = 0;
-    auth_freestate(popd_authstate);
-    popd_authstate = NULL;
     return 1;
 }
 
