@@ -1,6 +1,6 @@
 /* quota.c -- program to report/reconstruct quotas
  *
- * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,7 +39,8 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  */
-/* $Id: quota.c,v 1.44 2003/01/08 17:40:17 rjs3 Exp $ */
+
+/* $Id: quota.c,v 1.54 2004/05/22 03:45:52 rjs3 Exp $ */
 
 
 #include <config.h>
@@ -56,7 +57,6 @@
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <sys/stat.h>
-#include <com_err.h>
 
 #if HAVE_DIRENT_H
 # include <dirent.h>
@@ -76,14 +76,17 @@
 #endif
 
 #include "assert.h"
-#include "imapconf.h"
+#include "cyrusdb.h"
+#include "global.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "xmalloc.h"
 #include "mboxlist.h"
 #include "mboxname.h"
+#include "quota.h"
 #include "convert_code.h"
+#include "util.h"
 
 extern int optind;
 extern char *optarg;
@@ -91,18 +94,26 @@ extern char *optarg;
 /* current namespace */
 static struct namespace quota_namespace;
 
+/* config.c stuff */
+const int config_need_data = CONFIG_NEED_PARTITION_DATA;
+
 /* forward declarations */
 void usage(void);
 void reportquota(void);
-int buildquotalist(char **roots, int nroots);
+int buildquotalist(char *domain, char **roots, int nroots);
 int fixquota_mailbox(char *name,
 		     int matchlen,
 		     int maycreate,
-		     void* rock);
-int fixquota(int ispartial);
+		     void *rock);
+int fixquota(char *domain, int ispartial);
 int fixquota_fixroot(struct mailbox *mailbox,
 		     char *root);
-int fixquota_finish(int thisquota);
+int fixquota_finish(int thisquota, struct txn **tid);
+
+struct fix_rock {
+    char *domain;
+    struct txn **tid;
+};
 
 struct quotaentry {
     struct quota quota;
@@ -113,7 +124,6 @@ struct quotaentry {
 
 #define QUOTAGROW 300
 
-struct quotaentry zeroquotaentry;
 struct quotaentry *quota;
 int quota_num = 0, quota_alloc = 0;
 
@@ -126,16 +136,20 @@ int main(int argc,char **argv)
     int opt;
     int fflag = 0;
     int r, code = 0;
-    char *alt_config = NULL;
+    char *alt_config = NULL, *domain = NULL;
 
     if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
 
-    while ((opt = getopt(argc, argv, "C:f")) != EOF) {
+    while ((opt = getopt(argc, argv, "C:d:f")) != EOF) {
 	switch (opt) {
 	case 'C': /* alt config file */
 	    alt_config = optarg;
 	    break;
 
+	case 'd':
+	    domain = optarg;
+	    break;
+
 	case 'f':
 	    fflag = 1;
 	    break;
@@ -145,7 +159,7 @@ int main(int argc,char **argv)
 	}
     }
 
-    config_init(alt_config, "quota");
+    cyrus_init(alt_config, "quota", 0);
 
     /* Set namespace -- force standard (internal) */
     if ((r = mboxname_init_namespace(&quota_namespace, 1)) != 0) {
@@ -153,14 +167,20 @@ int main(int argc,char **argv)
 	fatal(error_message(r), EC_CONFIG);
     }
 
-    r = buildquotalist(argv+optind, argc-optind);
+    quotadb_init(0);
+    quotadb_open(NULL);
+
+    if (!r) r = buildquotalist(domain, argv+optind, argc-optind);
 
     if (!r && fflag) {
 	mboxlist_init(0);
-	r = fixquota(argc-optind);
+	r = fixquota(domain, argc-optind);
 	mboxlist_done();
     }
 
+    quotadb_close();
+    quotadb_done();
+
     if (!r) reportquota();
 
     if (r) {
@@ -168,37 +188,84 @@ int main(int argc,char **argv)
 	code = convert_code(r);
     }
 
-    exit(code);
+    cyrus_done();
 
-    return 0;
+    return code;
 }
 
 void usage(void)
 {
-    fprintf(stderr, "usage: quota [-C <alt_config>] [-f] [prefix]...\n");
+    fprintf(stderr, "usage: quota [-C <alt_config>] [-d <domain>] [-f] [prefix]...\n");
     exit(EC_USAGE);
 }    
 
-/*
- * Comparison function for sorting quota roots
- */
-int compare_quota(const void *a, const void *b)
+struct find_rock {
+    char **roots;
+    int nroots;
+};
+
+static int find_p(void *rockp,
+		  const char *key, int keylen,
+		  const char *data __attribute__((unused)),
+		  int datalen __attribute__((unused)))
 {
-    return strcmp(((struct quotaentry *)a)->quota.root,
-		      ((struct quotaentry *)b)->quota.root);
+    struct find_rock *frock = (struct find_rock *) rockp;
+    int i;
+
+    /* If restricting our list, see if this quota root matches */
+    if (frock->nroots) {
+	const char *p;
+
+	/* skip over domain */
+	if (config_virtdomains && (p = strchr(key, '!'))) {
+	    keylen -= (++p - key);
+	    key = p;
+	}
+
+ 	for (i = 0; i < frock->nroots; i++) {
+	    if (keylen >= strlen(frock->roots[i]) &&
+		!strncmp(key, frock->roots[i], strlen(frock->roots[i]))) {
+		break;
+	    }
+	}
+	if (i == frock->nroots) return 0;
+    }
+
+    return 1;
+}
+
+/*
+ * Add a quota root to the list in 'quota'
+ */
+static int find_cb(void *rockp __attribute__((unused)),
+		   const char *key, int keylen,
+		   const char *data, int datalen __attribute__((unused)))
+{
+    if (!data) return 0;
+
+    if (quota_num == quota_alloc) {
+	quota_alloc += QUOTAGROW;
+	quota = (struct quotaentry *)
+	  xrealloc((char *)quota, quota_alloc * sizeof(struct quotaentry));
+    }
+    memset(&quota[quota_num], 0, sizeof(struct quotaentry));
+    quota[quota_num].quota.root = xstrndup(key, keylen);
+    sscanf(data, "%lu %d",
+	   &quota[quota_num].quota.used, &quota[quota_num].quota.limit);
+  
+    quota_num++;
+
+    return 0;
 }
 
 /*
  * Build the list of quota roots in 'quota'
  */
-int buildquotalist(char **roots, int nroots)
+int buildquotalist(char *domain, char **roots, int nroots)
 {
-    int r;
-    char quota_path[MAX_MAILBOX_PATH];
     int i;
-    DIR *dirp;
-    DIR *topp;
-    struct dirent *dirent;
+    char buf[MAX_MAILBOX_NAME+1];
+    struct find_rock frock = { roots, nroots };
 
     /* Translate separator in mailboxnames.
      *
@@ -208,71 +275,17 @@ int buildquotalist(char **roots, int nroots)
      * translated name.
      */
     for (i = 0; i < nroots; i++) {
-	mboxname_hiersep_tointernal(&quota_namespace, roots[i]);
+	mboxname_hiersep_tointernal(&quota_namespace, roots[i], 0);
     }
 
-    snprintf(quota_path, sizeof(quota_path),
-	     "%s%s", config_dir, FNAME_QUOTADIR);
-    if (chdir(quota_path)) {
-	return IMAP_IOERROR;
-    }
-    
-    topp = opendir(".");
-    if (!topp) {
-	return IMAP_IOERROR;
-    }
-    while ((dirent = readdir(topp))!=NULL) {
-	if (dirent->d_name[0] == '.') continue;
-	
-	dirp = opendir(dirent->d_name);
-	if (!dirp) continue;
+    buf[0] = '\0';
+    if (domain) snprintf(buf, sizeof(buf), "%s!", domain);
 
-	while ((dirent = readdir(dirp))!=NULL) {
-	    if (dirent->d_name[0] == '.') continue;
-
-	    /* If restricting our list, see if this quota file matches */
-	    if (nroots) {
-		for (i = 0; i < nroots; i++) {
-		    if (!strcmp(dirent->d_name, roots[i]) ||
-			(!strncmp(dirent->d_name, roots[i], strlen(roots[i])) &&
-			 dirent->d_name[strlen(roots[i])] == '.')) break;
-		}
-		if (i == nroots) continue;
-	    }
+    /* if we have exactly one root specified, narrow the search */
+    if (nroots == 1) strlcat(buf, roots[0], sizeof(buf));
 	    
-	    /* Ignore .NEW files */
-	    i = strlen(dirent->d_name);
-	    if (i > 4 && !strcmp(dirent->d_name+i-4, ".NEW")) continue;
-	    
-	    if (quota_num == quota_alloc) {
-		quota_alloc += QUOTAGROW;
-		quota = (struct quotaentry *)
-		    xrealloc((char *)quota, quota_alloc * sizeof(struct quotaentry));
-	    }
-	    quota[quota_num] = zeroquotaentry;
-	    quota[quota_num].quota.fd = -1;
-	    quota[quota_num].quota.root = xstrdup(dirent->d_name);
-	    
-	    r = mailbox_read_quota(&quota[quota_num].quota);
-	    if (quota[quota_num].quota.fd != -1) {
-		close(quota[quota_num].quota.fd);
-		quota[quota_num].quota.fd = -1;
-	    }
-	    if (r) {
-		com_err(dirent->d_name, r,
-			(r == EC_IOERR) ? error_message(errno) : NULL);
-		quota[quota_num].quota.used = 0;
-		quota[quota_num].quota.limit = -1;
-	    }
-	    
-	    quota_num++;
-	}
-	
-	/* close this subdirectory */
-	closedir(dirp);
-    }
-    closedir(topp);
-    qsort((char *)quota, quota_num, sizeof(*quota), compare_quota);
+    config_quota_db->foreach(qdb, buf, strlen(buf),
+			     &find_p, &find_cb, &frock, NULL);
 
     return 0;
 }
@@ -281,18 +294,30 @@ int buildquotalist(char **roots, int nroots)
  * Account for mailbox 'name' when fixing the quota roots
  */
 int fixquota_mailbox(char *name,
-		     int matchlen,
-		     int maycreate,
+		     int matchlen __attribute__((unused)),
+		     int maycreate __attribute__((unused)),
 		     void* rock)
 {
     int r;
     struct mailbox mailbox;
     int i, len, thisquota, thisquotalen;
+    struct fix_rock *frock = (struct fix_rock *) rock;
+    char *p, *domain = frock->domain;
+    struct txn **tid = frock->tid;
+
+    /* make sure the domains match */
+    if ((p = strchr(name, '!')) != NULL) {
+	if (!domain || (p - name) != strlen(domain) ||
+	    strncmp(name, domain, p - name)) {
+	  return 0;
+	}
+    }
+    else if (domain) return 0;
 
     while (firstquota < quota_num &&
 	   strncmp(name, quota[firstquota].quota.root,
 		       strlen(quota[firstquota].quota.root)) > 0) {
-	r = fixquota_finish(firstquota++);
+	r = fixquota_finish(firstquota++, tid);
 	if (r) return r;
     }
 
@@ -302,7 +327,8 @@ int fixquota_mailbox(char *name,
 	 i < quota_num && strcmp(name, quota[i].quota.root) >= 0; i++) {
 	len = strlen(quota[i].quota.root);
 	if (!strncmp(name, quota[i].quota.root, len) &&
-	    (!name[len] || name[len] == '.')) {
+	    (!name[len] || name[len] == '.' ||
+	     (domain && name[len-1] == '!'))) {
 	    quota[i].refcount++;
 	    if (len > thisquotalen) {
 		thisquota = i;
@@ -337,14 +363,6 @@ int fixquota_mailbox(char *name,
 	}
     }
     
-    if (quota[thisquota].quota.fd == -1) {
-	r = mailbox_lock_quota(&quota[thisquota].quota);
-	if (r) {
-	    mailbox_close(&mailbox);
-	    return r;
-	}
-    }
-
     r = mailbox_open_index(&mailbox);
     if (r) {
 	mailbox_close(&mailbox);
@@ -360,24 +378,8 @@ int fixquota_mailbox(char *name,
 int fixquota_fixroot(struct mailbox *mailbox,
 		     char *root)
 {
-    int i, r;
+    int r;
 
-    /*
-     * Locking order is to lock header before quota.  We therefore
-     * unlock all the quota roots we have locked in order to avoid a
-     * deadlock.  As releasing these locks can cause the quota use
-     * recalculation to screw up, we set the global variable 'redofix'
-     * to cause the quota use recalculation to be redone.
-     *
-     * We could optimize this by trying to get a nonblocking lock on
-     * the header and unlocking all the quota roots only when that fails.
-     */
-    for (i = firstquota; i < quota_num; i++) {
-	if (quota[i].quota.fd != -1) {
-	    close(quota[i].quota.fd);
-	    quota[i].quota.fd = -1;
-	}
-    }
     redofix = 1;
 
     r = mailbox_lock_header(mailbox);
@@ -403,43 +405,29 @@ int fixquota_fixroot(struct mailbox *mailbox,
 /*
  * Finish fixing up a quota root
  */
-int fixquota_finish(int thisquota)
+int fixquota_finish(int thisquota, struct txn **tid)
 {
     int r;
 
     if (!quota[thisquota].refcount) {
 	if (!quota[thisquota].deleted++) {
-	    char buf[MAX_MAILBOX_PATH+1];
-	    
 	    printf("%s: removed\n", quota[thisquota].quota.root);
-	    mailbox_hash_quota(buf, sizeof(buf), quota[thisquota].quota.root);
-
-	    unlink(buf);
+	    r = quota_delete(&quota[thisquota].quota, tid);
+	    if (r) return r;
+	    free(quota[thisquota].quota.root);
+	    quota[thisquota].quota.root = NULL;
 	}
 	return 0;
     }
 
-    if (quota[thisquota].quota.fd == -1) {
-	r = mailbox_lock_quota(&quota[thisquota].quota);
-	if (r) {
-	    if (quota[thisquota].quota.fd != -1) {
-		close(quota[thisquota].quota.fd);
-		quota[thisquota].quota.fd = -1;
-	    }
-	    return r;
-	}
-    }
-    
     if (quota[thisquota].quota.used != quota[thisquota].newused) {
 	printf("%s: usage was %lu, now %lu\n", quota[thisquota].quota.root,
 	       quota[thisquota].quota.used, quota[thisquota].newused);
 	quota[thisquota].quota.used = quota[thisquota].newused;
-	r = mailbox_write_quota(&quota[thisquota].quota);
+	r = quota_write(&quota[thisquota].quota, tid);
 	if (r) return r;
     }
 
-    close(quota[thisquota].quota.fd);
-    quota[thisquota].quota.fd = -1;
     return 0;
 }
 
@@ -447,10 +435,12 @@ int fixquota_finish(int thisquota)
 /*
  * Fix all the quota roots
  */
-int fixquota(int ispartial)
+int fixquota(char *domain, int ispartial)
 {
     int r = 0;
     static char pattern[2] = "*";
+    struct fix_rock frock;
+    struct txn *tid = NULL;
 
     /*
      * Lock mailbox list to prevent mailbox creation/deletion
@@ -458,6 +448,9 @@ int fixquota(int ispartial)
      */
     mboxlist_open(NULL);
 
+    frock.domain = domain;
+    frock.tid = &tid;
+
     redofix = 1;
     while (redofix) {
 	redofix = 0;
@@ -465,20 +458,22 @@ int fixquota(int ispartial)
 	partial = ispartial;
 
 	r = (*quota_namespace.mboxlist_findall)(&quota_namespace, pattern, 1,
-						0, 0, fixquota_mailbox, NULL);
+						0, 0, fixquota_mailbox, &frock);
 	if (r) {
 	    mboxlist_close();
 	    return r;
 	}
 
 	while (firstquota < quota_num) {
-	    r = fixquota_finish(firstquota++);
+	    r = fixquota_finish(firstquota++, &tid);
 	    if (r) {
 		mboxlist_close();
 		return r;
 	    }
 	}
     }
+
+    if (!r) quota_commit(&tid);
     
     mboxlist_close();
     return 0;
@@ -491,7 +486,7 @@ void
 reportquota(void)
 {
     int i;
-    char buf[MAX_MAILBOX_PATH];
+    char buf[MAX_MAILBOX_PATH+1];
 
     printf("   Quota  %% Used    Used Root\n");
 
@@ -514,9 +509,3 @@ reportquota(void)
 	printf(" %7ld %s\n", quota[i].quota.used / QUOTA_UNITS, buf);
     }
 }
-
-void fatal(const char* s, int code)
-{
-    fprintf(stderr, "quota: %s\n", s);
-    exit(code);
-}
