@@ -36,10 +36,11 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- * $Id: version.h,v 1.126 2003/08/15 19:34:42 rjs3 Exp $
+ *
+ * $Id: version.h,v 1.132 2004/05/27 16:40:09 rjs3 Exp $
  */
 
-#define _CYRUS_VERSION "v2.1.15"
+#define _CYRUS_VERSION "v2.2.5"
 
 /* EXTRA_IDENT is a hack to add some version information for which compile
  * was used to build this version (at CMU, but we don't care what you do with
@@ -56,7 +57,10 @@
 #define CAPABILITY_STRING "IMAP4 IMAP4rev1 ACL QUOTA LITERAL+ " \
 	"MAILBOX-REFERRALS NAMESPACE UIDPLUS ID " \
 	"NO_ATOMIC_RENAME UNSELECT " \
-	"CHILDREN MULTIAPPEND SORT THREAD=ORDEREDSUBJECT THREAD=REFERENCES"
+	"CHILDREN MULTIAPPEND BINARY " \
+	"SORT THREAD=ORDEREDSUBJECT THREAD=REFERENCES " \
+	"ANNOTATEMORE"
+
 
 /* Values for ID processing */
 enum {
