@@ -1,7 +1,7 @@
 /* imtest.c -- IMAP/POP3/NNTP/LMTP/SMTP/MUPDATE/MANAGESIEVE test client
  * Ken Murchison (multi-protocol implementation)
  * Tim Martin (SASL implementation)
- * $Id: imtest.c,v 1.104 2004/12/07 19:26:22 ken3 Exp $
+ * $Id: imtest.c,v 1.98 2004/05/02 00:11:13 ken3 Exp $
  *
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
@@ -181,7 +181,7 @@ struct capa_cmd_t {
     char *resp;		/* end of capability response */
     char *tls;		/* [OPTIONAL] TLS capability string */
     char *auth;		/* [OPTIONAL] AUTH capability string */
-    char *(*parse_mechlist)(const char *str, struct protocol_t *prot);
+    char *(*parse_mechlist)(char *str, struct protocol_t *prot);
 			/* [OPTIONAL] parse capability string,
 			   returns space-separated list of mechs */
 };
@@ -625,15 +625,13 @@ static long bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi,
 	return (ret);
     
     if (cmd == (BIO_CB_READ | BIO_CB_RETURN)) {
-	printf("read from %08X [%08lX] (%d bytes => %ld (0x%X))\n",
-	       (unsigned int) bio, (long unsigned int) argp,
-	       argi, ret, (unsigned int) ret);
+	printf("read from %08X [%08lX] (%d bytes => %ld (0x%X))\n", bio, argp,
+	       argi, ret, ret);
 	tls_dump(argp, (int) ret);
 	return (ret);
     } else if (cmd == (BIO_CB_WRITE | BIO_CB_RETURN)) {
-	printf("write to %08X [%08lX] (%d bytes => %ld (0x%X))\n",
-	       (unsigned int) bio, (long unsigned int) argp,
-	       argi, ret, (unsigned int) ret);
+	printf("write to %08X [%08lX] (%d bytes => %ld (0x%X))\n", bio, argp,
+	       argi, ret, ret);
 	tls_dump(argp, (int) ret);
     }
     return (ret);
@@ -1019,7 +1017,6 @@ int auth_sasl(struct sasl_cmd_t *sasl_cmd, char *mechlist)
     int inbase64len;
     char cmdbuf[40];
     int sendliteral = sasl_cmd->quote;
-    int initial_response = 1;
     imt_stat status;
     
     if (!sasl_cmd || !sasl_cmd->cmd) return IMTEST_FAIL;
@@ -1065,8 +1062,6 @@ int auth_sasl(struct sasl_cmd_t *sasl_cmd, char *mechlist)
 	    printf(" ");
 	    prot_printf(pout, " ");
 	}
-    } else {
-	goto noinitresp;
     }
 
     do {
@@ -1082,21 +1077,12 @@ int auth_sasl(struct sasl_cmd_t *sasl_cmd, char *mechlist)
 		prot_printf(pout, "{%d+}\r\n", inbase64len);
 		prot_flush(pout);
 	    }
-	    printf("%s%s", initial_response ? "" : "C: ", inbase64);
+	    printf("%s", inbase64);
 	    prot_write(pout, inbase64, inbase64len);
 
 	    out = NULL;
-	} else if(sendliteral) {
-	    /* If we had no response, we still need to send the
-	       empty literal in this case */
-	    printf("{0+}\r\nC: ");
-	    prot_printf(pout, "{0+}\r\n");
-	} else {
-	    printf("C: ");
 	}
       noinitresp:
-	initial_response = 0;
-	
 	printf("\r\n");
 	prot_printf(pout, "\r\n");
 	prot_flush(pout);
@@ -1127,6 +1113,7 @@ int auth_sasl(struct sasl_cmd_t *sasl_cmd, char *mechlist)
 	    return saslresult;
 	}
 
+	if (status == STAT_CONT) printf("C: ");
 	sendliteral = !sasl_cmd->cont;
 
     } while (status == STAT_CONT);
@@ -1567,28 +1554,38 @@ static int generic_pipe(char *buf, int len, void *rock)
  *
  */
 
-static char *imap_parse_mechlist(const char *str, struct protocol_t *prot)
+static char *imap_parse_mechlist(char *str, struct protocol_t *prot)
 {
-    char *ret = xzmalloc(strlen(str)+1);
     char *tmp;
     int num = 0;
+    char *ret = malloc(strlen(str)+1);
+
+    if (ret == NULL) return NULL;
+    
+    strcpy(ret, "");
     
     if (strstr(str, "SASL-IR")) {
 	/* server supports initial response in AUTHENTICATE command */
 	prot->sasl_cmd.maxlen = INT_MAX;
     }
     
-    while ((tmp = strstr(str, " AUTH="))) {
-	char *end = (tmp += 6);
+    while ((tmp = strstr(str,"AUTH=")) != NULL) {
+	char *end = tmp+5;
+	tmp += 5;
 	
-	while((*end != ' ') && (*end != '\0')) end++;
+	while(((*end) != ' ') && ((*end) != '\0'))
+	    end++;
+	
+	(*end)='\0';
 	
 	/* add entry to list */
-	if (num++ > 0) strcat(ret, " ");
-	strlcat(ret, tmp, strlen(ret) + (end - tmp) + 1);
+	if (num > 0)
+	    strcat(ret, " ");
+	strcat(ret, tmp);
+	num++;
 	
 	/* reset the string */
-	str = end + 1;
+	str = end+1;
     }
     
     return ret;
@@ -2037,12 +2034,10 @@ static int nntp_do_auth(struct sasl_cmd_t *sasl_cmd,
 
 static char *nntp_parse_success(char *str)
 {
-    char *success = NULL, *tmp;
+    char *success = NULL;
 
     if (!strncmp(str, "283 ", 4)) {
 	success = str+4;
-	if ((tmp = strchr(success, ' ')))
-	    *tmp = '\0'; /* clip trailing comment */
     }
 
     return success;
@@ -2192,7 +2187,7 @@ void usage(char *prog, char *prot)
 static struct protocol_t protocols[] = {
     { "imap", "imaps", "imap",
       { 0, "* OK", NULL },
-      { "C01 CAPABILITY", "C01 ", " STARTTLS", " AUTH=", &imap_parse_mechlist },
+      { "C01 CAPABILITY", "C01 ", "STARTTLS", "AUTH=", &imap_parse_mechlist },
       { "S01 STARTTLS", "S01 OK", "S01 NO", 0 },
       { "A01 AUTHENTICATE", 0, 0, "A01 OK", "A01 NO", "+ ", "*", NULL },
       &imap_do_auth, { "Q01 LOGOUT", "Q01 " },
@@ -2205,11 +2200,11 @@ static struct protocol_t protocols[] = {
       { "AUTH", 255, 0, "+OK", "-ERR", "+ ", "*", NULL },
       &pop3_do_auth, { "QUIT", "+OK" }, NULL, NULL, NULL
     },
-    { "nntp", "nntps", "nntp",
+    { "nntp", "nntps", "news",
       { 0, "20", NULL },
-      { "CAPABILITIES", ".", "STARTTLS", "SASL ", NULL },
+      { "LIST EXTENSIONS", ".", "STARTTLS", "SASL ", NULL },
       { "STARTTLS", "382", "580", 0 },
-      { "AUTHINFO SASL", 512, 0, "28", "48", "383 ", "*", &nntp_parse_success },
+      { "AUTHINFO SASL", 512, 0, "28", "5", "383 ", "*", &nntp_parse_success },
       &nntp_do_auth, { "QUIT", "205" }, NULL, NULL, NULL
     },
     { "lmtp", NULL, "lmtp",
