@@ -59,9 +59,7 @@ int main(int argc, char *argv[])
     int txnp = 0;
     int r;
 
-    printf("Initing enviornment in '.'...\n");
     TRY(DB->init(".", 0));
-    printf("Ready!\n");
 
     for (;;) {
 	if (fgets(buf, sizeof buf, stdin) == NULL) break;
@@ -72,7 +70,7 @@ int main(int argc, char *argv[])
 	    if (db) { /* close it */
 		TRY(DB->close(db));
 	    }
-	    TRY(DB->open(fname, 1, &db));
+	    TRY(DB->open(fname, &db));
 
 	    printf("ok\n");
 	} else if (!db) {
