@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: master.c,v 1.100 2004/12/17 16:32:24 ken3 Exp $ */
+/* $Id: master.c,v 1.97 2004/05/18 19:16:50 rjs3 Exp $ */
 
 #include <config.h>
 
@@ -121,10 +121,6 @@ static int pidfd = -1;
 
 const char *MASTER_CONFIG_FILENAME = DEFAULT_MASTER_CONFIG_FILENAME;
 
-#define SERVICE_NONE -1
-#define SERVICE_MAX  INT_MAX-10
-#define SERVICENAME(x) ((x) ? x : "unknown")
-
 struct service *Services = NULL;
 int allocservices = 0;
 int nservices = 0;
@@ -156,7 +152,7 @@ struct centry {
     pid_t pid;
     enum sstate service_state;	/* SERVICE_STATE_* */
     time_t janitor_deadline;	/* cleanup deadline */
-    int si;			/* Services[] index */
+    struct service *s;
     struct centry *next;
 };
 static struct centry *ctable[child_table_size];
@@ -346,11 +342,7 @@ void service_create(struct service *s)
 
     if (s->associate > 0)
 	return;			/* service is already activated */
-
-    if (!s->name)
-	fatal("Serious software bug found: service_create() called on unnamed service!",
-		EX_SOFTWARE);
-
+    
     if (s->listen[0] == '/') { /* unix socket */
 	res0_is_local = 1;
 	res0 = (struct addrinfo *)malloc(sizeof(struct addrinfo));
@@ -386,22 +378,18 @@ void service_create(struct service *s)
  	} else if (!strcmp(s->proto, "tcp4")) {
  	    hints.ai_family = PF_INET;
  	    hints.ai_socktype = SOCK_STREAM;
-#ifdef PF_INET6
  	} else if (!strcmp(s->proto, "tcp6")) {
  	    hints.ai_family = PF_INET6;
  	    hints.ai_socktype = SOCK_STREAM;
-#endif
  	} else if (!strcmp(s->proto, "udp")) {
  	    hints.ai_family = PF_UNSPEC;
  	    hints.ai_socktype = SOCK_DGRAM;
  	} else if (!strcmp(s->proto, "udp4")) {
  	    hints.ai_family = PF_INET;
  	    hints.ai_socktype = SOCK_DGRAM;
-#ifdef PF_INET6 
-	} else if (!strcmp(s->proto, "udp6")) {
+ 	} else if (!strcmp(s->proto, "udp6")) {
  	    hints.ai_family = PF_INET6;
  	    hints.ai_socktype = SOCK_DGRAM;
-#endif
  	} else {
   	    syslog(LOG_INFO, "invalid proto '%s', disabling %s",
 		   s->proto, s->name);
@@ -502,8 +490,6 @@ void service_create(struct service *s)
 	
 	if (s == &service) {
 	    if (nservices == allocservices) {
-		if (allocservices > SERVICE_MAX - 5)
-		    fatal("out of service structures, please restart", EX_UNAVAILABLE);
 		Services = xrealloc(Services, 
 				    (allocservices+=5) * sizeof(struct service));
 		if (!Services) fatal("out of memory", EX_UNAVAILABLE);
@@ -582,7 +568,7 @@ void fcntl_unset(int fd, int flag)
     }
 }
 
-void spawn_service(const int si)
+void spawn_service(struct service *s)
 {
     /* Note that there is logic that depends on this being 2 */
     const int FORKRATE_INTERVAL = 2;
@@ -592,18 +578,12 @@ void spawn_service(const int si)
     char path[PATH_MAX];
     static char name_env[100], name_env2[100];
     struct centry *c;
-    struct service * const s = &Services[si];
     time_t now = time(NULL);
-
-    if (!s->name) {
-	fatal("Serious software bug found: spawn_service() called on unnamed service!",
-		EX_SOFTWARE);
-    }
-
+    
     /* update our fork rate */
     if(now - s->last_interval_start >= FORKRATE_INTERVAL) {
 	int interval;
-
+	
 	s->forkrate = (s->interval_forks/2) + (s->forkrate/2);
 	s->interval_forks = 0;
 	s->last_interval_start += FORKRATE_INTERVAL;
@@ -697,7 +677,7 @@ void spawn_service(const int si)
 	c = get_centry();
 	c->pid = p;
 	c->service_state = SERVICE_STATE_READY;
-	c->si = si;
+	c->s = s;
 	c->next = ctable[p % child_table_size];
 	ctable[p % child_table_size] = c;
 	break;
@@ -709,10 +689,6 @@ void schedule_event(struct event *a)
 {
     struct event *ptr;
 
-    if (! a->name)
-	fatal("Serious software bug found: schedule_event() called on unnamed event!",
-		EX_SOFTWARE);
-
     if (!schedule || a->mark < schedule->mark) {
 	a->next = schedule;
 	schedule = a;
@@ -791,7 +767,7 @@ void spawn_schedule(time_t now)
 		c = get_centry();
 		c->pid = p;
 		c->service_state = SERVICE_STATE_READY;
-		c->si = SERVICE_NONE;
+		c->s = NULL;
 		c->next = ctable[p % child_table_size];
 		ctable[p % child_table_size] = c;
 		
@@ -825,7 +801,6 @@ void reap_child(void)
     int status;
     pid_t pid;
     struct centry *c;
-    struct service *s;
 
     while ((pid = waitpid((pid_t) -1, &status, WNOHANG)) > 0) {
 	if (WIFEXITED(status)) {
@@ -843,8 +818,6 @@ void reap_child(void)
 	while(c && c->pid != pid) c = c->next;
 	
 	if (c && c->pid == pid) {
-	    s = ((c->si) != SERVICE_NONE) ? &Services[c->si] : NULL;
-
 	    /* paranoia */
 	    switch (c->service_state) {
 	    case SERVICE_STATE_READY:
@@ -855,23 +828,23 @@ void reap_child(void)
 	    default:
 		syslog(LOG_CRIT, 
 		       "service %s pid %d in ILLEGAL STATE: exited. Serious software bug or memory corruption detected!",
-		       SERVICENAME(s->name), pid);
+		       ((c->s) ? c->s->name : "unknown"), pid);
 		syslog(LOG_DEBUG,
 		       "service %s pid %d in ILLEGAL state: forced to valid UNKNOWN state",
-		       SERVICENAME(s->name), pid);
+		       ((c->s) ? c->s->name : "unknown"), pid);
 		c->service_state = SERVICE_STATE_UNKNOWN;
 	    }
-	    if (s) {
+	    if (c->s) {
 	        /* update counters for known services */
 		switch (c->service_state) {
 		case SERVICE_STATE_READY:
-		    s->nactive--;
-		    s->ready_workers--;
+		    c->s->nactive--;
+		    c->s->ready_workers--;
 		    if (WIFSIGNALED(status) ||
 			(WIFEXITED(status) && WEXITSTATUS(status))) {
 			syslog(LOG_WARNING, 
 			       "service %s pid %d in READY state: terminated abnormally",
-			       SERVICENAME(s->name), pid);
+			       c->s->name, pid);
 		    }
 		    break;
 		    
@@ -879,34 +852,35 @@ void reap_child(void)
 		    /* uh? either we got duplicate signals, or we are now MT */
 		    syslog(LOG_WARNING, 
 			   "service %s pid %d in DEAD state: receiving duplicate signals", 
-			   SERVICENAME(s->name), pid);
+			   c->s->name, pid);
 		    break;
 		    
 		case SERVICE_STATE_BUSY:
-		    s->nactive--;
+		    c->s->nactive--;
 		    if (WIFSIGNALED(status) ||
 			(WIFEXITED(status) && WEXITSTATUS(status))) {
 			syslog(LOG_DEBUG,
 			       "service %s pid %d in BUSY state: terminated abnormally",
-			       SERVICENAME(s->name), pid);
+			       c->s->name, pid);
 		    }
 		    break;
 		    
 		case SERVICE_STATE_UNKNOWN:
-		    s->nactive--;
+		    c->s->nactive--;
 		    syslog(LOG_WARNING,
 			   "service %s pid %d in UNKNOWN state: exited",
-			   SERVICENAME(s->name), pid);
+			   c->s->name, pid);
+		    break;
+
+		default:
+		    /* Prevent Warning */
 		    break;
 		} 
 	    } else {
-	    	/* children from spawn_schedule (events) or
-		 * children that messaged us before being registered or
-		 * children of services removed by reread_conf() */
+	    	/* children from spawn_schedule (events) or children that messaged us before being registered */
 		if (c->service_state != SERVICE_STATE_READY) {
-		    syslog(LOG_WARNING,
-			   "unknown service pid %d in state %d: exited (maybe using a service as an event,"
-			   " or a service was removed by SIGHUP?)",
+		    syslog(LOG_ERR,
+			   "unknown service pid %d in state %d: exited (maybe using a service as an event?)",
 			   pid, c->service_state);
 		}
 	    }
@@ -921,14 +895,13 @@ void reap_child(void)
 	    c->pid = pid;
 	    c->service_state = SERVICE_STATE_DEAD;
 	    c->janitor_deadline = time(NULL) + 2;
-	    c->si = SERVICE_NONE;
+	    c->s = NULL;
 	    c->next = ctable[pid % child_table_size];
 	    ctable[pid % child_table_size] = c;
 	}
-	if (verbose && c && (c->si != SERVICE_NONE))
-	    syslog(LOG_DEBUG, "service %s now has %d ready workers\n", 
-		    SERVICENAME(Services[c->si].name),
-		    Services[c->si].ready_workers);
+    if (verbose && c && c->s)
+	syslog(LOG_DEBUG, "service %s now has %d ready workers\n", 
+	       c->s->name, c->s->ready_workers);
     }
 }
 
@@ -1075,11 +1048,14 @@ void sighandler_setup(void)
     }
 }
 
-void process_msg(const int si, struct notify_message *msg) 
+void process_msg(struct service *s, struct notify_message *msg) 
 {
-    struct centry *c;
-    /* si must NOT point to an invalid service */
-    struct service * const s = &Services[si];;
+    struct centry * c;
+    
+    /* s must NOT be null
+     * but we don't assert(s) since the current code 
+     * makes NULL s an impossibility anyway 
+     */
 
     /* Search hash table with linked list for pid */
     c = ctable[msg->service_pid % child_table_size];
@@ -1088,10 +1064,10 @@ void process_msg(const int si, struct notify_message *msg)
     /* Did we find it? */
     if (!c || c->pid != msg->service_pid) {
 	syslog(LOG_WARNING, "service %s pid %d: while trying to process message 0x%x: not registered yet", 
-	       SERVICENAME(s->name), msg->service_pid, msg->message);
+	       s->name, msg->service_pid, msg->message);
 	/* resilience paranoia. Causes small performance loss when used */
 	c = get_centry();
-	c->si = si;
+	c->s = s;
 	c->pid = msg->service_pid;
 	c->service_state = SERVICE_STATE_UNKNOWN;
 	c->next = ctable[c->pid % child_table_size];
@@ -1099,18 +1075,17 @@ void process_msg(const int si, struct notify_message *msg)
     }
     
     /* paranoia */
-    if (si != c->si) {
+    if (s != c->s) {
 	syslog(LOG_ERR, 
 	       "service %s pid %d: changing from service %s due to received message",
-	       SERVICENAME(s->name), c->pid,
-	       ((c->si != SERVICE_NONE && Services[c->si].name) ? Services[c->si].name : "unknown"));
-	c->si = si;
+	       s->name, c->pid, ( (c->s) ? c->s->name : "unknown" ));
+	c->s = s;
     }
     switch (c->service_state) {
     case SERVICE_STATE_UNKNOWN:
 	syslog(LOG_WARNING, 
 	       "service %s pid %d in UNKNOWN state: processing message 0x%x",
-	       SERVICENAME(s->name), c->pid, msg->message);
+	       s->name, c->pid, msg->message);
 	break;
     case SERVICE_STATE_READY:
     case SERVICE_STATE_BUSY:
@@ -1119,10 +1094,10 @@ void process_msg(const int si, struct notify_message *msg)
     default:
 	syslog(LOG_CRIT,
 	       "service %s pid %d in ILLEGAL state: detected. Serious software bug or memory corruption uncloaked while processing message 0x%x from child!",
-	       SERVICENAME(s->name), c->pid, msg->message);
+	       s->name, c->pid, msg->message);
 	syslog(LOG_DEBUG,
 	       "service %s pid %d in ILLEGAL state: forced to valid UNKNOWN state",
-	       SERVICENAME(s->name), c->pid);
+	       s->name, c->pid);
 	c->service_state = SERVICE_STATE_UNKNOWN;
 	break;
     }
@@ -1135,7 +1110,7 @@ void process_msg(const int si, struct notify_message *msg)
 	    /* duplicate message? */
 	    syslog(LOG_WARNING,
 		   "service %s pid %d in READY state: sent available message but it is already ready",
-		   SERVICENAME(s->name), c->pid);
+		   s->name, c->pid);
 	    break;
 	    
 	case SERVICE_STATE_UNKNOWN:
@@ -1143,7 +1118,7 @@ void process_msg(const int si, struct notify_message *msg)
 	     * we don't increment ready_workers */
 	    syslog(LOG_DEBUG,
 		   "service %s pid %d in UNKNOWN state: now available and in READY state",
-		   SERVICENAME(s->name), c->pid);
+		   s->name, c->pid);
 	    c->service_state = SERVICE_STATE_READY;
 	    break;
 	    
@@ -1151,10 +1126,14 @@ void process_msg(const int si, struct notify_message *msg)
 	    if (verbose) 
 		syslog(LOG_DEBUG,
 		       "service %s pid %d in BUSY state: now available and in READY state",
-		       SERVICENAME(s->name), c->pid);
+		       s->name, c->pid);
 	    c->service_state = SERVICE_STATE_READY;
 	    s->ready_workers++;
 	    break;
+
+	default:
+	    /* Prevent Warning */
+	    break;
 	}
 	break;
 
@@ -1164,13 +1143,13 @@ void process_msg(const int si, struct notify_message *msg)
 	    /* duplicate message? */
 	    syslog(LOG_WARNING,
 		   "service %s pid %d in BUSY state: sent unavailable message but it is already busy",
-		   SERVICENAME(s->name), c->pid);
+		   s->name, c->pid);
 	    break;
 	    
 	case SERVICE_STATE_UNKNOWN:
 	    syslog(LOG_DEBUG,
 		   "service %s pid %d in UNKNOWN state: now unavailable and in BUSY state",
-		   SERVICENAME(s->name), c->pid);
+		   s->name, c->pid);
 	    c->service_state = SERVICE_STATE_BUSY;
 	    break;
 	    
@@ -1178,10 +1157,14 @@ void process_msg(const int si, struct notify_message *msg)
 	    if (verbose)
 		syslog(LOG_DEBUG,
 		       "service %s pid %d in READY state: now unavailable and in BUSY state",
-		       SERVICENAME(s->name), c->pid);
+		       s->name, c->pid);
 	    c->service_state = SERVICE_STATE_BUSY;
 	    s->ready_workers--;
 	    break;
+
+	default:
+	    /* Prevent Warning */
+	    break;
 	}
 	break;
 
@@ -1192,7 +1175,7 @@ void process_msg(const int si, struct notify_message *msg)
 	    if (verbose)
 		syslog(LOG_DEBUG,
 		       "service %s pid %d in BUSY state: now serving connection",
-		       SERVICENAME(s->name), c->pid);
+		       s->name, c->pid);
 	    break;
 	    
 	case SERVICE_STATE_UNKNOWN:
@@ -1200,18 +1183,22 @@ void process_msg(const int si, struct notify_message *msg)
 	    c->service_state = SERVICE_STATE_BUSY;
 	    syslog(LOG_DEBUG,
 		   "service %s pid %d in UNKNOWN state: now in BUSY state and serving connection",
-		   SERVICENAME(s->name), c->pid);
+		   s->name, c->pid);
 	    break;
 	    
 	case SERVICE_STATE_READY:
 	    syslog(LOG_ERR, 
 		   "service %s pid %d in READY state: reported new connection, forced to BUSY state",
-		   SERVICENAME(s->name), c->pid);
+		   s->name, c->pid);
 	    /* be resilient on face of a bogon source, so lets err to the side
 	     * of non-denial-of-service */
 	    c->service_state = SERVICE_STATE_BUSY;
 	    s->nconnections++;
 	    s->ready_workers--;
+
+	default:
+	    /* Prevent Warning */
+	    break;
 	}
 	break;
 	
@@ -1222,13 +1209,13 @@ void process_msg(const int si, struct notify_message *msg)
 	    if (verbose)
 		syslog(LOG_DEBUG, 
 		       "service %s pid %d in READY state: serving one more multi-threaded connection",
-		       SERVICENAME(s->name), c->pid);
+		       s->name, c->pid);
 	    break;
 	    
 	case SERVICE_STATE_BUSY:
 	    syslog(LOG_ERR, 
 		   "service %s pid %d in BUSY state: serving one more multi-threaded connection, forced to READY state",
-		   SERVICENAME(s->name), c->pid);
+		   s->name, c->pid);
 	    /* be resilient on face of a bogon source, so lets err to the side
 	     * of non-denial-of-service */
 	    c->service_state = SERVICE_STATE_READY;
@@ -1241,20 +1228,24 @@ void process_msg(const int si, struct notify_message *msg)
 	    c->service_state = SERVICE_STATE_READY;
 	    syslog(LOG_ERR,
 		   "service %s pid %d in UNKNOWN state: serving one more multi-threaded connection, forced to READY state",
-		   SERVICENAME(s->name), c->pid);
+		   s->name, c->pid);
+	    break;
+
+	default:
+	    /* Prevent Warning */
 	    break;
 	}
 	break;
 	
     default:
 	syslog(LOG_CRIT, "service %s pid %d: Software bug: unrecognized message 0x%x", 
-	       SERVICENAME(s->name), c->pid, msg->message);
+	       s->name, c->pid, msg->message);
 	break;
     }
 
     if (verbose)
 	syslog(LOG_DEBUG, "service %s now has %d ready workers\n", 
-	       SERVICENAME(s->name), s->ready_workers);
+	       s->name, s->ready_workers);
 }
 
 static char **tokenize(char *p)
@@ -1313,7 +1304,6 @@ void add_service(const char *name, struct entry *e, void *rock)
     char *proto = xstrdup(masterconf_getstring(e, "proto", "tcp"));
     char *max = xstrdup(masterconf_getstring(e, "maxchild", "-1"));
     rlim_t maxfds = (rlim_t) masterconf_getint(e, "maxfds", 256);
-    int reconfig = 0;
     int i, j;
 
     if(babysit && prefork == 0) prefork = 1;
@@ -1332,16 +1322,11 @@ void add_service(const char *name, struct entry *e, void *rock)
 	fatal(buf, EX_CONFIG);
     }
 
-    /* see if we have an existing entry that can be reused */
+    /* see if we have an existing entry for this service */
     for (i = 0; i < nservices; i++) {
-	/* skip non-primary instances */
 	if (Services[i].associate > 0)
 	    continue;
-	/* must have empty/same service name, listen and proto */
-	if ((!Services[i].name || !strcmp(Services[i].name, name)) &&
-	    (!Services[i].listen || !strcmp(Services[i].listen, listen)) &&
-	    (!Services[i].proto || !strcmp(Services[i].proto, proto)))
-	    break;
+	if (Services[i].name && !strcmp(Services[i].name, name)) break;
     }
 
     /* we have duplicate service names in the config file */
@@ -1356,66 +1341,47 @@ void add_service(const char *name, struct entry *e, void *rock)
 
 	fatal(buf, EX_CONFIG);
     }
- 
-    if (i == nservices) {
-	/* either we don't have an existing entry or we are changing
-	 * the port parameters, so create a new service
-	 */
-	if (nservices == allocservices) {
-	    if (allocservices > SERVICE_MAX - 5)
-		fatal("out of service structures, please restart", EX_UNAVAILABLE);
-	    Services = xrealloc(Services, 
-			       (allocservices+=5) * sizeof(struct service));
+
+    if ((i < nservices) &&
+	!strcmp(Services[i].listen, listen) &&
+	!strcmp(Services[i].proto, proto)) {
+
+	/* we found an existing entry and the port paramters are the same */
+	Services[i].exec = tokenize(cmd);
+	if (!Services[i].exec) fatal("out of memory", EX_UNAVAILABLE);
+
+	/* is this service actually there? */
+	if (!verify_service_file(Services[i].exec)) {
+	    char buf[1024];
+	    snprintf(buf, sizeof(buf),
+		     "cannot find executable for service '%s'", name);
+
+	    /* if it is not, we're misconfigured, die. */
+	    fatal(buf, EX_CONFIG);
 	}
-	memset(&Services[nservices++], 0, sizeof(struct service));
 
-	Services[i].last_interval_start = time(NULL);
-    }
-    else if (Services[i].listen) reconfig = 1;
+	Services[i].maxforkrate = maxforkrate;
 
-    if (!Services[i].name) Services[i].name = xstrdup(name);
-    if (Services[i].listen) free(Services[i].listen);
-    Services[i].listen = listen;
-    if (Services[i].proto) free(Services[i].proto);
-    Services[i].proto = proto;
+ 	Services[nservices].maxfds = maxfds;
 
-    Services[i].exec = tokenize(cmd);
-    if (!Services[i].exec) fatal("out of memory", EX_UNAVAILABLE);
-
-    /* is this service actually there? */
-    if (!verify_service_file(Services[i].exec)) {
-	char buf[1024];
-	snprintf(buf, sizeof(buf),
-		 "cannot find executable for service '%s'", name);
-	
-	/* if it is not, we're misconfigured, die. */
-	fatal(buf, EX_CONFIG);
-    }
-
-    Services[i].maxforkrate = maxforkrate;
-    Services[i].maxfds = maxfds;
-
-    if (!strcmp(Services[i].proto, "tcp") ||
-	!strcmp(Services[i].proto, "tcp4") ||
-	!strcmp(Services[i].proto, "tcp6")) {
-	Services[i].desired_workers = prefork;
-	Services[i].babysit = babysit;
-	Services[i].max_workers = atoi(max);
-	if (Services[i].max_workers == -1) {
-	    Services[i].max_workers = INT_MAX;
+	if (!strcmp(Services[i].proto, "tcp") ||
+	    !strcmp(Services[i].proto, "tcp4") ||
+	    !strcmp(Services[i].proto, "tcp6")) {
+	    Services[i].desired_workers = prefork;
+	    Services[i].babysit = babysit;
+	    Services[i].max_workers = atoi(max);
+	    if (Services[i].max_workers == -1) {
+		Services[i].max_workers = INT_MAX;
+	    }
+	} else {
+	    /* udp */
+	    if (prefork > 1) prefork = 1;
+	    Services[i].desired_workers = prefork;
+	    Services[i].max_workers = 1;
 	}
-    } else {
-	/* udp */
-	if (prefork > 1) prefork = 1;
-	Services[i].desired_workers = prefork;
-	Services[i].max_workers = 1;
-    }
-    free(max);
  
-    if (reconfig) {
-	/* reconfiguring an existing service, update any other instances */
 	for (j = 0; j < nservices; j++) {
-	    if (Services[j].associate > 0 && Services[j].listen &&
+	    if (Services[j].associate > 0 &&
 		Services[j].name && !strcmp(Services[j].name, name)) {
 		Services[j].maxforkrate = Services[i].maxforkrate;
 		Services[j].exec = Services[i].exec;
@@ -1424,16 +1390,85 @@ void add_service(const char *name, struct entry *e, void *rock)
 		Services[j].max_workers = Services[i].max_workers;
 	    }
 	}
+
+	if (verbose > 2)
+	    syslog(LOG_DEBUG, "reconfig: service '%s' (%s, %s:%s, %d, %d)",
+		   Services[i].name, cmd,
+		   Services[i].proto, Services[i].listen,
+		   Services[i].desired_workers,
+		   Services[i].max_workers);
+    }
+    else {
+	/* either we don't have an existing entry or we are changing
+	 * the port parameters, so create a new service
+	 */
+	if (nservices == allocservices) {
+	    Services = xrealloc(Services, 
+			       (allocservices+=5) * sizeof(struct service));
+	}
+
+	Services[nservices].name = xstrdup(name);
+	Services[nservices].listen = listen;
+	Services[nservices].proto = proto;
+	Services[nservices].exec = tokenize(cmd);
+	if (!Services[nservices].exec) fatal("out of memory", EX_UNAVAILABLE);
+
+	/* is this service actually there? */
+	if (!verify_service_file(Services[i].exec)) {
+	    char buf[1024];
+	    snprintf(buf, sizeof(buf),
+		     "cannot find executable for service '%s'", name);
+
+	    /* if it is not, we're misconfigured, die. */
+	    fatal(buf, EX_CONFIG);
+	}
+
+	Services[nservices].socket = 0;
+	Services[nservices].saddr = NULL;
+
+	Services[nservices].ready_workers = 0;
+
+ 	Services[nservices].maxfds = maxfds;
+	Services[nservices].maxforkrate = maxforkrate;
+
+	if(!strcmp(Services[nservices].proto, "tcp") ||
+	   !strcmp(Services[nservices].proto, "tcp4") ||
+	   !strcmp(Services[nservices].proto, "tcp6")) {
+	    Services[nservices].desired_workers = prefork;
+	    Services[nservices].babysit = babysit;
+	    Services[nservices].max_workers = atoi(max);
+	    if (Services[nservices].max_workers == -1) {
+		Services[nservices].max_workers = INT_MAX;
+	    }
+	} else {
+	    if (prefork > 1) prefork = 1;
+	    Services[nservices].desired_workers = prefork;
+	    Services[nservices].max_workers = 1;
+	}
+	
+	memset(Services[nservices].stat, 0, sizeof(Services[nservices].stat));
+
+	Services[nservices].last_interval_start = time(NULL);
+	Services[nservices].interval_forks = 0;
+	Services[nservices].forkrate = 0;
+	
+	Services[nservices].nforks = 0;
+	Services[nservices].nactive = 0;
+	Services[nservices].nconnections = 0;
+	Services[nservices].associate = 0;
+	
+	if (verbose > 2)
+	    syslog(LOG_DEBUG, "add: service '%s' (%s, %s:%s, %d, %d, %d)",
+		   Services[nservices].name, cmd,
+		   Services[nservices].proto, Services[nservices].listen,
+		   Services[nservices].desired_workers,
+		   Services[nservices].max_workers,
+		   (int) Services[nservices].maxfds);
+
+	nservices++;
     }
 
-    if (verbose > 2)
-	syslog(LOG_DEBUG, "%s: service '%s' (%s, %s:%s, %d, %d, %d)",
-	       reconfig ? "reconfig" : "add",
-	       Services[i].name, cmd,
-	       Services[i].proto, Services[i].listen,
-	       Services[i].desired_workers,
-	       Services[i].max_workers,
-	       (int) Services[i].maxfds);
+    free(max);
 }
 
 void add_event(const char *name, struct entry *e, void *rock)
@@ -1534,9 +1569,8 @@ void limit_fds(rlim_t x)
 
 void reread_conf(void)
 {
-    int i,j;
+    int i;
     struct event *ptr;
-    struct centry *c;
 
     /* disable all services -
        they will be re-enabled if they appear in config file */
@@ -1554,26 +1588,17 @@ void reread_conf(void)
 		       Services[i].name, Services[i].socket,
 		       Services[i].stat[0], Services[i].stat[1]);
 
-	    /* Only free the service info on the primary */
+	    /* Only free the service info once */
 	    if(Services[i].associate == 0) {
+		free(Services[i].name);
 		free(Services[i].listen);
 		free(Services[i].proto);
 	    }
-	    Services[i].listen = NULL;
-	    Services[i].proto = NULL;
+	    Services[i].name = NULL;
 	    Services[i].desired_workers = 0;
-
-	    /* send SIGHUP to all children */
-	    for (j = 0 ; j < child_table_size ; j++ ) {
-		c = ctable[j];
-		while (c != NULL) {
-		    if ((c->si == i) &&
-			(c->service_state != SERVICE_STATE_DEAD)) {
-			kill(c->pid, SIGHUP);
-		    }
-		    c = c->next;
-		}
-	    }
+	    Services[i].nforks = 0;
+	    Services[i].nactive = 0;
+	    Services[i].nconnections = 0;
 
 	    /* close all listeners */
 	    if (Services[i].socket > 0) {
@@ -1581,6 +1606,11 @@ void reread_conf(void)
 		close(Services[i].socket);
 	    }
 	    Services[i].socket = 0;
+	    Services[i].saddr = NULL;
+
+	    if (Services[i].stat[0] > 0) close(Services[i].stat[0]);
+	    if (Services[i].stat[1] > 0) close(Services[i].stat[1]);
+	    memset(Services[i].stat, 0, sizeof(Services[i].stat));
 	}
 	else if (Services[i].exec && !Services[i].socket) {
 	    /* initialize new services */
@@ -1593,6 +1623,7 @@ void reread_conf(void)
 	}
     }
 
+
     /* remove existing events */
     while (schedule) {
 	ptr = schedule;
@@ -1606,11 +1637,6 @@ void reread_conf(void)
 
     /* reinit child janitor */
     init_janitor();
-
-    /* send some feedback to admin */
-    syslog(LOG_NOTICE,
-	    "Services reconfigured. %d out of %d (max %d) services structures are now in use",
-	    nservices, allocservices, SERVICE_MAX);
 }
 
 int main(int argc, char **argv)
@@ -1949,7 +1975,7 @@ int main(int argc, char **argv)
 	    if (Services[i].exec /* enabled */ &&
 		(Services[i].nactive < Services[i].max_workers) &&
 		(Services[i].ready_workers < Services[i].desired_workers)) {
-		spawn_service(i);
+		spawn_service(&Services[i]);
 	    } else if (Services[i].exec
 		       && Services[i].babysit
 		       && Services[i].nactive == 0) {
@@ -1957,28 +1983,7 @@ int main(int argc, char **argv)
 		       "lost all children for service: %s.  " \
 		       "Applying babysitter.",
 		       Services[i].name);
-		spawn_service(i);
-	    } else if (!Services[i].exec /* disabled */ &&
-		       Services[i].name /* not yet removed */ &&
-		       Services[i].nactive == 0) {
-		if (verbose > 2)
-		    syslog(LOG_DEBUG, "remove: service %s pipe %d %d",
-			   Services[i].name,
-			   Services[i].stat[0], Services[i].stat[1]);
-
-		/* Only free the service info on the primary */
-		if (Services[i].associate == 0) {
-		    free(Services[i].name);
-		}
-		Services[i].name = NULL;
-		Services[i].nforks = 0;
-		Services[i].nactive = 0;
-		Services[i].nconnections = 0;
-		Services[i].associate = 0;
-
-		if (Services[i].stat[0] > 0) close(Services[i].stat[0]);
-		if (Services[i].stat[1] > 0) close(Services[i].stat[1]);
-		memset(Services[i].stat, 0, sizeof(Services[i].stat));
+		spawn_service(&Services[i]);
 	    }
 	}
 
@@ -2062,7 +2067,7 @@ int main(int argc, char **argv)
 		    continue;
 		}
 		
-		process_msg(i, &msg);
+		process_msg(&Services[i], &msg);
 	    }
 
 	    if (Services[i].exec &&
@@ -2072,13 +2077,13 @@ int main(int argc, char **argv)
 		     j < Services[i].desired_workers; 
 		     j++)
 		{
-		    spawn_service(i);
+		    spawn_service(&Services[i]);
 		}
 
 		if (Services[i].ready_workers == 0 && 
 		    FD_ISSET(y, &rfds)) {
 		    /* huh, someone wants to talk to us */
-		    spawn_service(i);
+		    spawn_service(&Services[i]);
 		}
 	    }
 	}
