@@ -1,4 +1,4 @@
-/* service.c -- skeleton for Cyrus service; calls the real main
+/* service-thread.c -- skeleton for Cyrus service; calls the real main
  *
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: service-thread.c,v 1.11 2003/05/29 02:10:55 rjs3 Exp $ */
+/* $Id: service-thread.c,v 1.17 2004/02/27 18:21:27 ken3 Exp $ */
 #include <config.h>
 
 #include <stdio.h>
@@ -66,6 +66,7 @@
 #include <string.h>
 
 #include "service.h"
+#include "xmalloc.h"
 
 extern int optind;
 extern char *optarg;
@@ -76,8 +77,11 @@ static int verbose = 0;
 
 void notify_master(int fd, int msg)
 {
-    if (verbose) syslog(LOG_DEBUG, "telling master %d", msg);
-    if (write(fd, &msg, sizeof(msg)) != sizeof(msg)) {
+    struct notify_message notifymsg;
+    if (verbose) syslog(LOG_DEBUG, "telling master %x", msg);
+    notifymsg.message = msg;
+    notifymsg.service_pid = getpid();
+    if (write(fd, &notifymsg, sizeof(notifymsg)) != sizeof(notifymsg)) {
 	syslog(LOG_ERR, "unable to tell master %x: %m", msg);
     }
 }
@@ -96,12 +100,12 @@ static void libwrap_init(struct request_info *r, char *service)
 static int libwrap_ask(struct request_info *r, int fd)
 {
     int a;
-    struct sockaddr_in sin;
+    struct sockaddr_storage sin;
     socklen_t len = sizeof(sin);
     
     /* is this a connection from the local host? */
     if (getpeername(fd, (struct sockaddr *) &sin, &len) == 0) {
-	if (sin.sin_family == AF_UNIX) {
+	if (((struct sockaddr *)&sin)->sa_family == AF_UNIX) {
 	    return 1;
 	}
     }
@@ -122,19 +126,21 @@ static int libwrap_ask(struct request_info *r, int fd)
 #else
 struct request_info { int x; };
 
-static void libwrap_init(struct request_info *r, char *service)
+static void libwrap_init(struct request_info *r __attribute__((unused)),
+			 char *service __attribute__((unused)))
 {
 
 }
 
-static int libwrap_ask(struct request_info *r, int fd)
+static int libwrap_ask(struct request_info *r __attribute__((unused)),
+		       int fd __attribute__((unused)))
 {
     return 1;
 }
 
 #endif
 
-extern void config_init(const char *, const char *);
+extern void cyrus_init(const char *, const char *, unsigned);
 extern const char *config_getstring(const char *key, const char *def);
 
 #define ARGV_GROW 10
@@ -149,18 +155,7 @@ int main(int argc, char **argv, char **envp)
     char *alt_config = NULL;
     int call_debugger = 0;
     int newargc = 0;
-    char **newargv = (char **) malloc(ARGV_GROW * sizeof(char *));
-
-    p = getenv("CYRUS_SERVICE");
-    if (p == NULL) {
-	syslog(LOG_ERR, "could not getenv(CYRUS_SERVICE); exiting");
-	exit(EX_SOFTWARE);
-    }
-    service = strdup(p);
-    if (service == NULL) {
-	syslog(LOG_ERR, "couldn't strdup() service: %m");
-	exit(EX_OSERR);
-    }
+    char **newargv = (char **) xmalloc(ARGV_GROW * sizeof(char *));
 
     opterr = 0; /* disable error reporting,
 		   since we don't know about service-specific options */
@@ -177,8 +172,8 @@ int main(int argc, char **argv, char **envp)
 	    break;
 	default:
 	    if (!((newargc+1) % ARGV_GROW)) { /* time to alloc more */
-		newargv = (char **) realloc(newargv, (newargc + ARGV_GROW) * 
-					    sizeof(char *));
+		newargv = (char **) xrealloc(newargv, (newargc + ARGV_GROW) * 
+					     sizeof(char *));
 	    }
 	    newargv[newargc++] = argv[optind-1];
 
@@ -189,14 +184,11 @@ int main(int argc, char **argv, char **envp)
 	    break;
 	}
     }
-
-    config_init(alt_config, service);
-
     /* grab the remaining arguments */
     for (; optind < argc; optind++) {
 	if (!(newargc % ARGV_GROW)) { /* time to alloc more */
-	    newargv = (char **) realloc(newargv, (newargc + ARGV_GROW) * 
-					sizeof(char *));
+	    newargv = (char **) xrealloc(newargv, (newargc + ARGV_GROW) * 
+					 sizeof(char *));
 	}
 	newargv[newargc++] = argv[optind];
     }
@@ -212,6 +204,15 @@ int main(int argc, char **argv, char **envp)
 	sleep(15);
     }
 
+    p = getenv("CYRUS_SERVICE");
+    if (p == NULL) {
+	syslog(LOG_ERR, "could not getenv(CYRUS_SERVICE); exiting");
+	exit(EX_SOFTWARE);
+    }
+    service = xstrdup(p);
+
+    cyrus_init(alt_config, service, 0);
+
     if (call_debugger) {
 	char debugbuf[1024];
 	int ret;
@@ -232,7 +233,8 @@ int main(int argc, char **argv, char **envp)
 				       fdflags | FD_CLOEXEC);
     if (fdflags == -1) {
 	syslog(LOG_ERR, "unable to set close on exec: %m");
-	notify_master(STATUS_FD, MASTER_SERVICE_UNAVAILABLE);
+	if (MESSAGE_MASTER_ON_EXIT) 
+	    notify_master(STATUS_FD, MASTER_SERVICE_UNAVAILABLE);
 	return 1;
     }
     fdflags = fcntl(STATUS_FD, F_GETFD, 0);
@@ -240,12 +242,14 @@ int main(int argc, char **argv, char **envp)
 				       fdflags | FD_CLOEXEC);
     if (fdflags == -1) {
 	syslog(LOG_ERR, "unable to set close on exec: %m");
-	notify_master(STATUS_FD, MASTER_SERVICE_UNAVAILABLE);
+	if (MESSAGE_MASTER_ON_EXIT) 
+	    notify_master(STATUS_FD, MASTER_SERVICE_UNAVAILABLE);
 	return 1;
     }
 
     if (service_init(newargc, newargv, envp) != 0) {
-	notify_master(STATUS_FD, MASTER_SERVICE_UNAVAILABLE);
+	if (MESSAGE_MASTER_ON_EXIT) 
+	    notify_master(STATUS_FD, MASTER_SERVICE_UNAVAILABLE);
 	return 1;
     }
 
@@ -269,10 +273,13 @@ int main(int argc, char **argv, char **envp)
 		case EOPNOTSUPP:
 		case ENETUNREACH:
 		case EAGAIN:
+		case EINTR:
+		case ECONNABORTED:
 		    break;
 		default:
 		    syslog(LOG_ERR, "accept failed: %m");
-		    notify_master(STATUS_FD, MASTER_SERVICE_UNAVAILABLE);
+		    if (MESSAGE_MASTER_ON_EXIT) 
+			notify_master(STATUS_FD, MASTER_SERVICE_UNAVAILABLE);
 		    service_abort(EX_OSERR);
 		}
 	    }
@@ -290,13 +297,14 @@ int main(int argc, char **argv, char **envp)
 	syslog(LOG_DEBUG, "accepted connection");
 
 	use_count++;
-	notify_master(STATUS_FD, MASTER_SERVICE_CONNECTION);
+	notify_master(STATUS_FD, MASTER_SERVICE_CONNECTION_MULTI);
 	if (service_main_fd(fd, newargc, newargv, envp) < 0) {
 	    break;
 	}
     }
 
-    notify_master(STATUS_FD, MASTER_SERVICE_UNAVAILABLE);
+    if (MESSAGE_MASTER_ON_EXIT) 
+	notify_master(STATUS_FD, MASTER_SERVICE_UNAVAILABLE);
     service_abort(0);
     return 0;
 }
