@@ -39,7 +39,7 @@
 # OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #
 #
-# $Id: cyradm.sh,v 1.17 2005/11/21 16:39:24 murch Exp $
+# $Id: cyradm.sh,v 1.16 2004/01/15 14:35:34 ken3 Exp $
 case "x$BASH_VERSION" in
 x) exec perl -MCyrus::IMAP::Shell -e shell -- ${1+"$@"} ;;
 *) exec perl -MCyrus::IMAP::Shell -e shell -- "$@" ;;
@@ -216,12 +216,6 @@ Sets a comment or description associated with the mailbox.
 
 Sets the number of days after which messages will be expired from the mailbox.
 
-=item C<sieve>
-
-Indicates the name of the global sieve script that should be run when
-a message is delivered to the shared mailbox (not used for personal
-mailboxes).
-
 =item C<squat>
 
 Indicates that the mailbox should have a squat index created for it.
