@@ -39,7 +39,7 @@
  *
  */
 
-/* $Id: isieve.c,v 1.29 2005/04/21 21:03:49 shadow Exp $ */
+/* $Id: isieve.c,v 1.27 2003/10/22 18:50:25 rjs3 Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -64,7 +64,6 @@
 #include "request.h"
 #include "iptostring.h"
 #include "xmalloc.h"
-#include "util.h"
 
 #include <prot.h>
 
@@ -115,7 +114,7 @@ int init_net(char *serverFQDN, int port, isieve_t **obj)
   struct addrinfo hints, *res0, *res;
   int err;
   char portstr[6];
-  int sock = -1;
+  int sock;
     
   snprintf(portstr, sizeof(portstr), "%d", port);
   memset(&hints, 0, sizeof(hints));
@@ -251,9 +250,7 @@ char * read_capability(isieve_t *obj)
   while (yylex(&state,obj->pin)==STRING)
   {
       char *attr = string_DATAPTR(state.str);
-      char *val;
-
-      val = NULL;
+      char *val = NULL;
 
       if (yylex(&state,obj->pin)==' ')
       {
@@ -261,7 +258,7 @@ char * read_capability(isieve_t *obj)
 	  {
 	      parseerror("STRING");
 	  }
-	  val = xstrdup(string_DATAPTR(state.str));
+	  val = string_DATAPTR(state.str);
 	  if (yylex(&state,obj->pin)!=EOL)
 	  {
 	      parseerror("EOL1");
@@ -270,8 +267,7 @@ char * read_capability(isieve_t *obj)
 
       if (strcasecmp(attr,"SASL")==0)
       {
-	if (cap) free(cap);
-	cap = xstrdup(val);
+	cap = val;
       } else if (strcasecmp(attr,"SIEVE")==0) {
 
       } else if (strcasecmp(attr,"IMPLEMENTATION")==0) {
@@ -282,13 +278,12 @@ char * read_capability(isieve_t *obj)
 	  obj->version = OLD_VERSION;
 	  cap = (char *) xmalloc(strlen(val));
 	  memset(cap, '\0', strlen(val));
-	  memcpy(cap, val+5, strlen(val)-6);
+	  memcpy(cap, val+6, strlen(val)-7);
 
 	  return cap;
       } else {
-	  /* unknown capability */
+	  /* unkown capability */
       }
-      if (val) { free(val); val = NULL; }
   }
 
   if (yylex(&state,obj->pin)!=EOL)
@@ -590,34 +585,7 @@ int do_referral(isieve_t *obj, char *refer_to)
     /* Authenticate */
     mechlist = read_capability(obj_new);
 
-    do {
-	mtried = NULL;
-	ret = auth_sasl(mechlist, obj_new, &mtried, &errstr);
-	if(ret) init_sasl(obj_new, 128, callbacks);
-
-	if(mtried) {
-	    char *newlist = (char*) xmalloc(strlen(mechlist)+1);
-	    char *mtr = (char*) xstrdup(mtried);
-	    char *tmp;
-	    
-	    ucase(mtr);
-	    tmp = strstr(mechlist,mtr);
-	    *tmp ='\0';
-	    strcpy(newlist, mechlist);
-	    tmp++;
-	    
-	    tmp = strchr(tmp,' ');
-	    if (tmp) {
-		strcat(newlist,tmp);
-	    }
-	    
-	    free(mtr);
-	    free(mechlist);
-	    mechlist = newlist;
-	}
-    } while(ret && mtried);
-
-    /* xxx leak? */
+    ret = auth_sasl(mechlist, obj_new, &mtried, &errstr);
     if(ret) return STAT_NO;
 
     /* free old isieve_t */
