@@ -1,5 +1,5 @@
 /* bc_eval.c - evaluate the bytecode
- * $Id: bc_eval.c,v 1.7 2004/07/29 15:42:31 rjs3 Exp $
+ * $Id: bc_eval.c,v 1.2 2003/10/22 18:03:24 rjs3 Exp $
  */
 /***********************************************************
         Copyright 2001 by Carnegie Mellon University
@@ -152,7 +152,7 @@ static char* look_for_me(char *myaddr, int numaddresses,
 	while (!found &&
 	       ((addr = get_address(ADDRESS_ALL,&data, &marker, 1))!= NULL)) {
 
-	    if (!strcasecmp(addr, myaddr)) {
+	    if (!strcmp(addr, myaddr)) {
 		found = xstrdup(myaddr);
 		break;
 	    }
@@ -172,7 +172,7 @@ static char* look_for_me(char *myaddr, int numaddresses,
 
 		altaddr = get_address(ADDRESS_ALL, &altdata, &altmarker, 1);
 
-		if (!strcasecmp(addr,altaddr)) {
+		if (!strcmp(addr,altaddr)) {
 		    found=xstrdup(str);
 		    break;
 		}
@@ -210,19 +210,6 @@ int shouldRespond(void * m, sieve_interp_t *interp,
 	while (*body[0] && isspace((int) *body[0])) body[0]++;
 	if (strcasecmp(body[0], "no")) l = SIEVE_DONE;
     }
-
-    /* is there a Precedence keyword of "junk | bulk | list"? */
-    strcpy(buf, "precedence");
-    if (interp->getheader(m, buf, &body) == SIEVE_OK) {
-	/* we don't deal with comments, etc. here */
-	/* skip leading white-space */
-	while (*body[0] && isspace((int) *body[0])) body[0]++;
-	if (!strcasecmp(body[0], "junk") ||
-	    !strcasecmp(body[0], "bulk") ||
-	    !strcasecmp(body[0], "list"))
-	    l = SIEVE_DONE;
-    }
-
     /* Note: the domain-part of all addresses are canonicalized */
     /* grab my address from the envelope */
     if (l == SIEVE_OK) {
@@ -974,7 +961,7 @@ int sieve_eval_bc(sieve_interp_t *i, const void *bc_in, unsigned int bc_len,
 	  
 	    res = do_notify(notify_list, id, method, options,
 			    priority, message);
-
+	 	  
 	    break;
 	}
 	case B_DENOTIFY:
